# -*- coding: utf-8 -*-

import sre_constants
from typing import Optional

from qtpy import QtCore
import regex as re

from prettyqt import gui


class RegexPatternValidator(gui.Validator):
    def __init__(self, parent: Optional[QtCore.QObject] = None):
        super().__init__(parent)
        self.error_message = ""
        self.compiled = None

    def __repr__(self):
        return "RegexPatternValidator()"

    def validate(self, text: str, pos: int = 0) -> tuple:
        # if text == "":
        #     self.compiled = None
        #     return (self.Intermediate, text, pos)
        try:
            self.compiled = re.compile(text)
        except sre_constants.error as e:
            self.error_message = str(e)
            self.compiled = None
            return (self.Intermediate, text, pos)
        except re._regex_core.error as e:
            self.error_message = str(e)
            self.compiled = None
            return (self.Intermediate, text, pos)
        else:
            self.error_message = ""
            return (self.Acceptable, text, pos)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    w = widgets.LineEdit()
    val = RegexPatternValidator()
    w.set_validator(val)
    w.show()
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtCore
from qtpy.QtCharts import QtCharts

from prettyqt import charts, core


STYLES = {
    QtCore.Qt.SolidLine: "Solid",
    QtCore.Qt.DotLine: "Dot",
    QtCore.Qt.DashDotLine: "Dash-dot",
}


QtCharts.QLineSeries.__bases__ = (charts.XYSeries,)


class LineSeries(QtCharts.QLineSeries):
    """QLineSeries with some custom properties."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._process_name = ""

    def get_process_name(self):
        return self._process_name

    def set_process_name(self, value):
        self._process_name = value

    process_name = core.Property(str, get_process_name, set_process_name)


if __name__ == "__main__":
    line = LineSeries()
    line.append(0, 1)



#############################################################################
##
# Copyright (C) 2007 Trolltech ASA. All rights reserved.
##
# This file is part of the Qt Concurrent project on Trolltech Labs.
##
# This file may be used under the terms of the GNU General Public
# License version 2.0 as published by the Free Software Foundation
# and appearing in the file LICENSE.GPL included in the packaging of
# this file.  Please review the following information to ensure GNU
# General Public Licensing requirements will be met:
# http://www.trolltech.com/products/qt/opensource.html
##
# If you are unsure which license is appropriate for your use, please
# review the following information:
# http://www.trolltech.com/products/qt/licensing.html or contact the
# sales department at sales@trolltech.com.
##
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
##
#############################################################################

from PyQt5 import QtCore, QtGui


# This was originally a Trolltech file.  The QBzr folks did some work to
# bring it up to date, and I've done some work on top of theirs to fix a few
# minor bugs.
#
# To test a model, instantiate this class with a reference to the model and
# its parent
#        from modeltest import ModelTest
#        model = MyFancyModel(self)
#        self.modeltest = ModelTest(model, self)


class ModelTest(QtCore.QObject):
    def __init__(self, _model, verbose=True, parent=None):
        """Connect to all of the models signals.

        Whenever anything happens check everything.
        """
        QtCore.QObject.__init__(self, parent)
        self._model = _model
        self.model = _model
        self.insert = []
        self.remove = []
        self.fetchingMore = False
        self._verbose = verbose
        assert self.model

        self.model.columnsAboutToBeInserted.connect(self.runAllTests)
        self.model.columnsAboutToBeRemoved.connect(self.runAllTests)
        self.model.columnsInserted.connect(self.runAllTests)
        self.model.columnsRemoved.connect(self.runAllTests)
        self.model.dataChanged.connect(self.runAllTests)
        self.model.headerDataChanged.connect(self.runAllTests)
        self.model.layoutAboutToBeChanged.connect(self.runAllTests)
        self.model.layoutChanged.connect(self.runAllTests)
        self.model.modelReset.connect(self.runAllTests)
        self.model.rowsAboutToBeInserted.connect(self.runAllTests)
        self.model.rowsAboutToBeRemoved.connect(self.runAllTests)
        self.model.rowsInserted.connect(self.runAllTests)
        self.model.rowsRemoved.connect(self.runAllTests)

        # Special checks for inserting/removing
        self.model.rowsAboutToBeInserted.connect(self.rowsAboutToBeInserted)
        self.model.rowsAboutToBeRemoved.connect(self.rowsAboutToBeRemoved)
        self.model.rowsInserted.connect(self.rowsInserted)
        self.model.rowsRemoved.connect(self.rowsRemoved)
        self.runAllTests()

    def nonDestructiveBasicTest(self):
        assert self.model.buddy(QtCore.QModelIndex()) == QtCore.QModelIndex()
        self.model.canFetchMore(QtCore.QModelIndex())
        assert self.model.columnCount(QtCore.QModelIndex()) >= 0
        assert (
            self.model.data(QtCore.QModelIndex(), QtCore.Qt.DisplayRole)
            == QtCore.QVariant()
        )
        self.fetchingMore = True
        self.model.fetchMore(QtCore.QModelIndex())
        self.fetchingMore = False
        flags = self.model.flags(QtCore.QModelIndex())
        assert (
            int(flags & QtCore.Qt.ItemIsEnabled) == QtCore.Qt.ItemIsEnabled
            or int(flags & QtCore.Qt.ItemIsEnabled) == 0
        )
        self.model.hasChildren(QtCore.QModelIndex())
        self.model.hasIndex(0, 0)
        self.model.headerData(0, QtCore.Qt.Horizontal, QtCore.Qt.DisplayRole)
        self.model.index(0, 0, QtCore.QModelIndex())
        self.model.itemData(QtCore.QModelIndex())
        cache = QtCore.QVariant()
        self.model.match(QtCore.QModelIndex(), -1, cache)
        self.model.mimeTypes()
        assert self.model.parent(QtCore.QModelIndex()) == QtCore.QModelIndex()
        assert self.model.rowCount(QtCore.QModelIndex()) >= 0
        variant = QtCore.QVariant()
        self.model.setData(QtCore.QModelIndex(), variant, -1)
        self.model.setHeaderData(-1, QtCore.Qt.Horizontal, QtCore.QVariant())
        self.model.setHeaderData(0, QtCore.Qt.Horizontal, QtCore.QVariant())
        self.model.setHeaderData(999999, QtCore.Qt.Horizontal, QtCore.QVariant())
        self.model.sibling(0, 0, QtCore.QModelIndex())
        self.model.span(QtCore.QModelIndex())
        self.model.supportedDropActions()

    def rowCount(self):
        """Test implementation of rowCount hasChildren method.

        self.models that are dynamically populated are not as fully tested here.
        """
        # check top row
        topindex = self.model.index(0, 0, QtCore.QModelIndex())
        rows = self.model.rowCount(topindex)
        assert rows >= 0
        if rows > 0:
            hasChildren = self.model.hasChildren(topindex)
            assert hasChildren is True

        secondlvl = self.model.index(0, 0, topindex)
        if secondlvl.isValid():
            # check a row count where parent is valid
            rows = self.model.rowCount(secondlvl)
            assert rows >= 0
            if rows > 0:
                assert self.model.hasChildren(secondlvl) is True

        # The self.models rowCount() is tested more extensively in checkChildren,
        # but this catches the big mistakes

    def columnCount(self):
        """Test implementation of columnCount and hasChildren method."""
        # check top row
        topidx = self.model.index(0, 0, QtCore.QModelIndex())
        assert self.model.columnCount(topidx) >= 0

        # check a column count where parent is valid
        childidx = self.model.index(0, 0, topidx)
        if childidx.isValid():
            assert self.model.columnCount(childidx) >= 0

        # columnCount() is tested more extensively in checkChildren,
        # but this catches the big mistakes

    def hasIndex(self):
        """Test implementation of hasIndex method."""
        # Make sure that invalid values returns an invalid index
        assert self.model.hasIndex(-2, -2) is False
        assert self.model.hasIndex(-2, 0) is False
        assert self.model.hasIndex(0, -2) is False

        rows = self.model.rowCount(QtCore.QModelIndex())
        cols = self.model.columnCount(QtCore.QModelIndex())

        # check out of bounds
        assert self.model.hasIndex(rows, cols) is False
        assert self.model.hasIndex(rows + 1, cols + 1) is False

        if rows > 0:
            assert self.model.hasIndex(0, 0) is True

        # hasIndex() is tested more extensively in checkChildren()
        # but this catches the big mistakes

    def index(self):
        """Test implementation of index method."""
        # Make sure that invalid values returns an invalid index
        assert self.model.index(-2, -2, QtCore.QModelIndex()) == QtCore.QModelIndex()
        assert self.model.index(-2, 0, QtCore.QModelIndex()) == QtCore.QModelIndex()
        assert self.model.index(0, -2, QtCore.QModelIndex()) == QtCore.QModelIndex()

        rows = self.model.rowCount(QtCore.QModelIndex())
        cols = self.model.columnCount(QtCore.QModelIndex())

        if rows == 0:
            return

        # Catch off by one errors
        assert self.model.index(rows, cols, QtCore.QModelIndex()) == QtCore.QModelIndex()
        assert self.model.index(0, 0, QtCore.QModelIndex()).isValid() is True

        # Make sure that the same index is *always* returned
        a = self.model.index(0, 0, QtCore.QModelIndex())
        b = self.model.index(0, 0, QtCore.QModelIndex())
        assert a == b

        # index() is tested more extensively in checkChildren()
        # but this catches the big mistakes

    def parent(self):
        """Test implementation of parent method."""
        # Make sure the self.model wont crash and will return an invalid QModelIndex
        # when asked for the parent of an invalid index
        assert self.model.parent(QtCore.QModelIndex()) == QtCore.QModelIndex()

        if self.model.rowCount(QtCore.QModelIndex()) == 0:
            return

        # Column 0              | Column 1  |
        # QtCore.Qself.modelIndex()         |           |
        #    \- topidx          | topidx1   |
        #         \- childix    | childidx1 |

        # Common error test #1, make sure that a top level index has a parent
        # that is an invalid QtCore.Qself.modelIndex
        topidx = self.model.index(0, 0, QtCore.QModelIndex())
        assert self.model.parent(topidx) == QtCore.QModelIndex()

        # Common error test #2, make sure that a second level index has a parent
        # that is the first level index
        if self.model.rowCount(topidx) > 0:
            childidx = self.model.index(0, 0, topidx)
            assert self.model.parent(childidx) == topidx

        # Common error test #3, the second column should NOT have the same children
        # as the first column in a row
        # Usually the second column shouldn't have children
        topidx1 = self.model.index(0, 1, QtCore.QModelIndex())
        if self.model.rowCount(topidx1) > 0:
            childidx = self.model.index(0, 0, topidx)
            childidx1 = self.model.index(0, 0, topidx1)
            assert childidx != childidx1

        # Full test, walk n levels deep through the self.model making sure that all
        # parent's children correctly specify their parent
        self.checkChildren(QtCore.QModelIndex())

    def data(self):
        """Test implementation of data method."""
        # Invalid index should return an invalid qvariant
        qvar = self.model.data(QtCore.QModelIndex(), QtCore.Qt.DisplayRole)
        assert qvar is None

        if self.model.rowCount(QtCore.QModelIndex()) == 0:
            return

        # A valid index should have a valid QtCore.QVariant data
        assert self.model.index(0, 0, QtCore.QModelIndex()).isValid()

        # shouldn't be able to set data on an invalid index
        assert (
            self.model.setData(
                QtCore.QModelIndex(), QtCore.QVariant("foo"), QtCore.Qt.DisplayRole
            )
            is False
        )

        # General Purpose roles that should return a QString
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.ToolTipRole
        )
        assert data is None or isinstance(data, str)
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.StatusTipRole
        )
        assert data is None or isinstance(data, str)
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.WhatsThisRole
        )
        assert data is None or isinstance(data, str)

        # General Purpose roles that should return a QSize
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.SizeHintRole
        )
        assert data is None or isinstance(data, QtCore.QSize)

        # General Purpose roles that should return a QFont
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.FontRole
        )
        assert data is None or isinstance(data, QtGui.QFont)

        # Check that the alignment is one we know about
        alignment = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.TextAlignmentRole
        )
        if alignment is not None:
            assert alignment == (
                alignment
                & int(QtCore.Qt.AlignHorizontal_Mask | QtCore.Qt.AlignVertical_Mask)
            )

        # General Purpose roles that should return a QColor
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.BackgroundColorRole
        )
        assert data is None or isinstance(data, QtGui.QColor)
        data = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.TextColorRole
        )
        assert data is None or isinstance(data, QtGui.QColor)

        # Check that the "check state" is one we know about.
        state = self.model.data(
            self.model.index(0, 0, QtCore.QModelIndex()), QtCore.Qt.CheckStateRole
        )
        assert state in [
            None,
            QtCore.Qt.Unchecked,
            QtCore.Qt.PartiallyChecked,
            QtCore.Qt.Checked,
        ]

    def runAllTests(self):
        if self.fetchingMore:
            return

        self.nonDestructiveBasicTest()
        print("passed nonDestructiveBasicTest") if self._verbose else None

        self.rowCount()
        print("passed rowCount") if self._verbose else None

        self.columnCount()
        print("passed columnCount") if self._verbose else None

        self.hasIndex()
        print("passed hasIndex") if self._verbose else None

        self.index()
        print("passed index") if self._verbose else None

        self.parent()
        print("passed parent") if self._verbose else None

        self.data()
        print("passed data") if self._verbose else None
        print("------------------------------") if self._verbose else None

    def rowsAboutToBeInserted(self, parent, start, end):
        """Store what is about to be inserted to make sure it actually happens."""
        c = {}
        c["parent"] = parent
        c["oldSize"] = self.model.rowCount(parent)
        c["last"] = self.model.data(self.model.index(start - 1, 0, parent))
        c["next"] = self.model.data(self.model.index(start, 0, parent))
        self.insert.append(c)

    def rowsInserted(self, parent, start, end):
        """Confirm that what was said was going to happen actually did."""
        c = self.insert.pop()
        assert c["parent"] == parent
        assert c["oldSize"] + (end - start + 1) == self.model.rowCount(parent)
        assert c["last"] == self.model.data(self.model.index(start - 1, 0, c["parent"]))

        # if c['next'] != self.model.data(model.index(end+1, 0, c['parent'])):
        #   qDebug << start << end
        #   for i in range(0, self.model.rowCount(QtCore.QModelIndex())):
        #       qDebug << self.model.index(i, 0).data().toString()
        #   qDebug() << c['next'] << self.model.data(model.index(end+1, 0, c['parent']))

        assert c["next"] == self.model.data(self.model.index(end + 1, 0, c["parent"]))

    def rowsAboutToBeRemoved(self, parent, start, end):
        """Store what is about to be inserted to make sure it actually happens."""
        c = {}
        c["parent"] = parent
        c["oldSize"] = self.model.rowCount(parent)
        c["last"] = self.model.data(self.model.index(start - 1, 0, parent))
        c["next"] = self.model.data(self.model.index(end + 1, 0, parent))
        self.remove.append(c)

    def rowsRemoved(self, parent, start, end):
        """Confirm that what was said was going to happen actually did."""
        c = self.remove.pop()
        assert c["parent"] == parent
        assert c["oldSize"] - (end - start + 1) == self.model.rowCount(parent)
        assert c["last"] == self.model.data(self.model.index(start - 1, 0, c["parent"]))
        assert c["next"] == self.model.data(self.model.index(start, 0, c["parent"]))

    def checkChildren(self, parent, depth=0):
        """Called from parent() test.

        A self.model that returns an index of parent X should also return X when asking
        for the parent of the index

        This recursive function does pretty extensive testing on the whole model in an
        effort to catch edge cases.

        This function assumes that rowCount(QtCore.QModelIndex()), columnCount
        (QtCore.QModelIndex()) and index() already work.
        If they have a bug it will point it out, but the above tests should have already
        found the basic bugs because it is easier to figure out the problem in
        those tests then this one
        """
        # First just try walking back up the tree.
        p = parent
        while p.isValid():
            p = p.parent()

        # For self.models that are dynamically populated
        if self.model.canFetchMore(parent):
            self.fetchingMore = True
            self.model.fetchMore(parent)
            self.fetchingMore = False

        rows = self.model.rowCount(parent)
        cols = self.model.columnCount(parent)

        if rows > 0:
            assert self.model.hasChildren(parent)

        # Some further testing against rows(), columns, and hasChildren()
        assert rows >= 0
        assert cols >= 0

        if rows > 0:
            assert self.model.hasChildren(parent) is True

        # qDebug() << "parent:" << self.model.data(parent).toString() << "rows:" << rows
        #          << "columns:" << cols << "parent column:" << parent.column()

        assert self.model.hasIndex(rows + 1, 0, parent) is False
        for r in range(0, rows):
            if self.model.canFetchMore(parent):
                self.fetchingMore = True
                self.model.fetchMore(parent)
                self.fetchingMore = False
            assert self.model.hasIndex(r, cols + 1, parent) is False
            for c in range(0, cols):
                assert self.model.hasIndex(r, c, parent)
                index = self.model.index(r, c, parent)
                # rowCount(index) and columnCount(index) said that it existed...
                assert index.isValid() is True

                # index() should always return the same index when called twice in a row
                modIdx = self.model.index(r, c, parent)
                assert index == modIdx

                # Make sure we get the same index if we request it twice in a row
                a = self.model.index(r, c, parent)
                b = self.model.index(r, c, parent)
                assert a == b

                # Some basic checking on the index that is returned
                assert index.model() == self._model
                assert index.row() == r
                assert index.column() == c
                # While you can technically return a QVariant usually this is a sign
                # if an bug in data() Disable if this really is ok in your self.model
                assert self.model.data(index, QtCore.Qt.DisplayRole) is not None

                # if the next test fails here is some somehwat useful debug you play with
                # if self.model.parent(index) != parent:
                #   qDebug() << r << c << depth << self.model.data(index).toString()
                #        << self.model.data(parent).toString()
                #   qDebug() << index << parent << self.model.parent(index)
                #   # And a view that you can even use to show the self.model
                #   # view = QtGui.QTreeView()
                #   # view.setself.model(model)
                #   # view.show()
                #

                # Check that we can get back our real parent
                p = self.model.parent(index)
                assert p.internalId() == parent.internalId()
                assert p.row() == parent.row()

                # recursively go down the children
                if self.model.hasChildren(index) and depth < 10:
                    # qDebug() << r << c << "hasChildren" << self.model.rowCount(index)
                    depth += 1
                    self.checkChildren(index, depth)
                # else:
                #   if depth >= 10:
                #       qDebug() << "checked 10 deep"

                # Make sure that after testing the children that the index doesn't change
                newIdx = self.model.index(r, c, parent)
                assert index == newIdx



# -*- coding: utf-8 -*-

import contextlib
from typing import Optional

from qtpy import QtGui, QtWidgets

from prettyqt import core, gui, widgets, syntaxhighlighters, constants
from prettyqt.gui import textcursor
from prettyqt.utils import bidict


WRAP_MODES = bidict(
    none=QtGui.QTextOption.NoWrap,
    word=QtGui.QTextOption.WordWrap,
    anywhere=QtGui.QTextOption.WrapAnywhere,
    boundary_or_anywhere=QtGui.QTextOption.WrapAtWordBoundaryOrAnywhere,
)

LINE_WRAP_MODES = bidict(
    none=QtWidgets.QPlainTextEdit.NoWrap,
    widget_width=QtWidgets.QPlainTextEdit.WidgetWidth,
)

MOVE_OPERATIONS = textcursor.MOVE_OPERATIONS
MOVE_MODES = textcursor.MOVE_MODES


QtWidgets.QPlainTextEdit.__bases__ = (widgets.AbstractScrollArea,)


class PlainTextEdit(QtWidgets.QPlainTextEdit):

    value_changed = core.Signal()

    def __init__(
        self,
        text: str = "",
        parent: Optional[QtWidgets.QWidget] = None,
        read_only: bool = False,
    ):
        super().__init__(text, parent)
        self._allow_wheel_zoom = False
        self.validator: Optional[QtGui.QValidator] = None
        self.textChanged.connect(self._on_value_change)
        self.set_read_only(read_only)

    def serialize_fields(self):
        return dict(
            text=self.text(), read_only=self.isReadOnly(), font=gui.Font(self.font()),
        )

    def __setstate__(self, state):
        self.__init__()
        self.set_text(state["text"])
        self.setEnabled(state.get("enabled", True))
        self.setFont(state["font"])
        self.setReadOnly(state["read_only"])

    def __add__(self, other):
        if isinstance(other, str):
            self.append_text(other)
            return self

    def wheelEvent(self, event):
        """Handle wheel event for zooming."""
        if not self._allow_wheel_zoom:
            return None
        if event.modifiers() & constants.CTRL_MOD:
            self.zoomIn() if event.angleDelta().y() > 0 else self.zoomOut()
        else:
            super().wheelEvent(event)

    def allow_wheel_zoom(self, do_zoom: bool = True):
        self._allow_wheel_zoom = do_zoom

    @contextlib.contextmanager
    def create_cursor(self):
        cursor = gui.TextCursor(self.document())
        yield cursor
        self.setTextCursor(cursor)

    @contextlib.contextmanager
    def current_cursor(self):
        cursor = gui.TextCursor(self.textCursor())
        yield cursor
        self.setTextCursor(cursor)

    def move_cursor(self, operation: str, mode: str = "move"):
        op = MOVE_OPERATIONS[operation]
        mode = MOVE_MODES[mode]
        self.moveCursor(op, mode)

    def append_text(self, text: str, newline: bool = True):
        if newline:
            self.appendPlainText(text)
        else:
            self.move_cursor("end")
            self.insertPlainText(text)
            self.move_cursor("end")

    def set_text(self, text: str):
        self.setPlainText(text)

    def set_syntaxhighlighter(self, syntax: str, style: Optional[str] = None):
        self._hl = syntaxhighlighters.PygmentsHighlighter(self.document(), syntax)
        if style is not None:
            self._hl.set_style(style)

    def text(self) -> str:
        return self.toPlainText()

    def select_text(self, start: int, end: int):
        with self.create_cursor() as c:
            c.select_text(start, end)

    def set_read_only(self, value: bool = True):
        """Make the PlainTextEdit read-only.

        Args:
            value: True, for read-only, otherwise False
        """
        self.setReadOnly(value)

    def highlight_current_line(self, color="yellow"):
        extra_selections = []

        if not self.isReadOnly():
            selection = widgets.TextEdit.ExtraSelection()
            line_color = gui.Color(color)
            selection.format.setBackground(line_color)
            selection.format.setProperty(QtGui.QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extra_selections.append(selection)

        self.setExtraSelections(extra_selections)

    def set_wrap_mode(self, mode: str):
        """Set word wrap mode.

        Allowed values are "none", "word", "anywhere", "boundary_or_anywhere"

        Args:
            mode: word wrap mode to use

        Raises:
            ValueError: wrap mode does not exist
        """
        if mode not in WRAP_MODES:
            raise ValueError(f"invalid wrap mode. Allowed values: {WRAP_MODES.keys()}")
        self.setWordWrapMode(WRAP_MODES[mode])

    def set_line_wrap_mode(self, mode: str):
        """Set line wrap mode.

        Allowed values are "none" and "widget width"

        Args:
            mode: line wrap mode to use

        Raises:
            ValueError: line wrap mode does not exist
        """
        if mode not in LINE_WRAP_MODES:
            raise ValueError(
                f"invalid wrap mode. " f"Allowed values: {LINE_WRAP_MODES.keys()}"
            )
        self.setLineWrapMode(LINE_WRAP_MODES[mode])

    def _on_value_change(self):
        self.value_changed.emit()
        if self.validator is not None:
            self._set_validation_color()

    def _set_validation_color(self, state: bool = True):
        color = "orange" if not self.is_valid() else None
        self.set_background_color(color)

    def set_validator(self, validator: Optional[QtGui.QValidator]):
        self.validator = validator
        self._set_validation_color()

    def set_regex_validator(self, regex: str, flags=0) -> gui.RegExpValidator:
        validator = gui.RegularExpressionValidator(self)
        validator.set_regex(regex, flags)
        self.set_validator(validator)
        return validator

    def is_valid(self):
        if self.validator is None:
            return True
        return self.validator.is_valid_value(self.text())

    def set_value(self, value: str):
        self.setPlainText(value)

    def get_value(self):
        return self.text()


if __name__ == "__main__":
    from prettyqt import custom_validators

    val = custom_validators.RegexPatternValidator()
    app = widgets.app()
    widget = PlainTextEdit("This is a test")
    widget.set_validator(val)
    with widget.current_cursor() as c:
        c.select_text(2, 4)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""Module containing custom delegate classes."""

from .buttondelegate import ButtonDelegate
from .radiodelegate import RadioDelegate
from .icondelegate import IconDelegate
from .nofocusdelegate import NoFocusDelegate

__all__ = [
    "ButtonDelegate",
    "RadioDelegate",
    "IconDelegate",
    "NoFocusDelegate",
]



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import widgets


QtWidgets.QTableWidget.__bases__ = (widgets.TableView,)


class TableWidget(QtWidgets.QTableWidget):
    def sort(self, column=0, reverse=False):
        order = QtCore.Qt.DescendingOrder if reverse else QtCore.Qt.AscendingOrder
        self.sortItems(column, order)

    def __getitem__(self, index):
        return self.item(*index)

    def __setitem__(self, index, value):
        print(index, value)
        self.setItem(index[0], index[1], value)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = TableWidget()
    widget.setHorizontalHeaderLabels(["testus"])
    widget.setColumnCount(1)
    widget.insertRow(0)
    widget[0, 0] = widgets.TableWidgetItem("test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets, QtCore

from prettyqt import core
from prettyqt.utils import bidict


COMPLETION_MODES = bidict(
    popup=QtWidgets.QCompleter.PopupCompletion,
    inline=QtWidgets.QCompleter.InlineCompletion,
    unfiltered_popup=QtWidgets.QCompleter.UnfilteredPopupCompletion,
)

SORT_MODES = bidict(
    unsorted=QtWidgets.QCompleter.UnsortedModel,
    case_sensitive=QtWidgets.QCompleter.CaseSensitivelySortedModel,
    case_insensitive=QtWidgets.QCompleter.CaseInsensitivelySortedModel,
)

FILTER_MODES = bidict(
    starts_with=QtCore.Qt.MatchStartsWith,
    contains=QtCore.Qt.MatchContains,
    ends_with=QtCore.Qt.MatchEndsWith,
)


QtWidgets.QCompleter.__bases__ = (core.Object,)


class Completer(QtWidgets.QCompleter):
    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(parent)

    def set_sort_mode(self, mode: Optional[str]):
        """Set sort mode to use.

        Allowed values are "unsorted", "case_sensitive", "case_insensitive"

        Args:
            mode: sort mode to use

        Raises:
            ValueError: sort mode does not exist
        """
        if mode is None:
            mode = "unsorted"
        if mode not in SORT_MODES:
            raise ValueError(f"Invalid sort mode. Valid values: {SORT_MODES.keys()}")
        self.setModelSorting(SORT_MODES[mode])

    def get_sort_mode(self) -> str:
        """Return current sort mode.

        Possible values: "unsorted", "case_sensitive", "case_insensitive"

        Returns:
            sort mode
        """
        return SORT_MODES.inv[self.modelSorting()]

    def set_completion_mode(self, mode: str):
        """Set completion mode to use.

        Allowed values are "popup", "inline", "unfiltered_popup"

        Args:
            mode: completion mode to use

        Raises:
            ValueError: completion mode does not exist
        """
        if mode not in COMPLETION_MODES:
            raise ValueError(f"Invalid mode. Valid values: {COMPLETION_MODES.keys()}")
        self.setCompletionMode(COMPLETION_MODES[mode])

    def get_completion_mode(self) -> str:
        """Return current completion mode.

        Possible values: "popup", "inline", "unfiltered_popup"

        Returns:
            completion mode
        """
        return COMPLETION_MODES.inv[self.completionMode()]

    def set_filter_mode(self, mode: str):
        """Set filter mode to use.

        Allowed values are "starts_with", "contains", "ends_with"

        Args:
            mode: filter mode to use

        Raises:
            ValueError: filter mode does not exist
        """
        if mode not in FILTER_MODES:
            raise ValueError(f"Invalid mode. Valid values: {FILTER_MODES.keys()}")
        self.setFilterMode(FILTER_MODES[mode])

    def get_filter_mode(self) -> str:
        """Return current filter mode.

        Possible values: "starts_with", "contains", "ends_with"

        Returns:
            filter mode
        """
        return FILTER_MODES.inv[self.filterMode()]


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    completer = Completer()
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtMultimediaWidgets

from prettyqt import multimedia, widgets


QtMultimediaWidgets.QVideoWidget.__bases__ = (
    widgets.Widget,
    multimedia.MediaBindableInterface,
)


class VideoWidget(QtMultimediaWidgets.QVideoWidget):
    pass



# -*- coding: utf-8 -*-

import contextlib

from qtpy import QtCore

from prettyqt import core


OPEN_MODES = core.iodevice.OPEN_MODES  # type: ignore

QtCore.QBuffer.__bases__ = (core.IODevice,)


class Buffer(QtCore.QBuffer):
    @contextlib.contextmanager
    def open_file(self, flag):
        if flag in OPEN_MODES:
            flag = OPEN_MODES[flag]
        self.open(flag)
        yield None
        self.close()



# -*- coding: utf-8 -*-

"""winextras module.

contains QtWinExtras-based classes
"""

try:
    from .winjumplistcategory import WinJumpListCategory
    from .winjumplistitem import WinJumpListItem
    from .winjumplist import WinJumpList

    __all__ = ["WinJumpListCategory", "WinJumpListItem", "WinJumpList"]
except (ImportError, ModuleNotFoundError):
    pass



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets
import regex as re

from prettyqt import core, custom_validators, custom_widgets, widgets


class RegexInput(widgets.Widget):

    value_changed = core.Signal()

    def __init__(
        self,
        show_flags: bool = True,
        show_error: bool = True,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent=parent)
        self.set_layout("grid")
        self.lineedit = custom_widgets.SingleLineTextEdit()
        self.lineedit.set_syntaxhighlighter("regex")
        self.tb_flags = custom_widgets.BoolDictToolButton(
            "Flags", icon="mdi.flag-variant-outline"
        )
        self.label_error = widgets.Label()
        self.label_error.set_color("red")
        self.box[0, 0:1] = self.lineedit
        if show_flags:
            self.box[0, 2] = self.tb_flags
        if show_error:
            self.box[1, 0:2] = self.label_error
        self.tb_flags.triggered.connect(self._on_value_change)
        self.lineedit.textChanged.connect(self._on_value_change)
        val = custom_validators.RegexPatternValidator()
        self.lineedit.set_validator(val)
        dct = {
            "multiline": "MultiLine",
            "ignorecase": "Ignore case",
            "ascii": "ASCII-only matching",
            "dotall": "Dot matches newline",
            "verbose": "Ignore whitespace",
        }
        self._mapping = {
            "ignorecase": re.IGNORECASE,
            "multiline": re.MULTILINE,
            "ascii": re.ASCII,
            "dotall": re.DOTALL,
            "verbose": re.VERBOSE,
        }
        self.tb_flags.set_dict(dct)

    def _on_value_change(self) -> None:
        self.value_changed.emit()
        if self.lineedit.is_valid():
            self.label_error.set_text("")
        elif self.lineedit.validator is not None:
            message = self.lineedit.validator.error_message
            self.label_error.set_text(message)

    @property
    def pattern(self) -> str:
        return self.lineedit.text()

    @pattern.setter
    def pattern(self, value: str):
        self.lineedit.set_text(value)

    @property
    def compile_flags(self) -> int:
        ret_val = 0
        for identifier, flag in self._mapping.items():
            if self.tb_flags[identifier]:
                ret_val |= flag
        return ret_val

    @compile_flags.setter
    def compile_flags(self, value: int):
        for identifier, flag in self._mapping.items():
            self.tb_flags[identifier] = bool(value & flag)

    def set_value(self, value):
        if value is None:
            self.pattern = ""
            self.flags = 0
        if isinstance(value, str):
            self.pattern = value
            self.flags = 0
        else:
            self.pattern = value.pattern
            self.compile_flags = value.flags

    def get_value(self):
        return re.compile(self.pattern, self.compile_flags)

    def is_valid(self) -> bool:
        return self.lineedit.is_valid()


if __name__ == "__main__":
    app = widgets.app()
    widget = RegexInput(show_flags=False, show_error=False)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict


SHADOWS = bidict(
    plain=QtWidgets.QFrame.Plain,
    raised=QtWidgets.QFrame.Raised,
    sunken=QtWidgets.QFrame.Sunken,
)

SHAPES = bidict(
    no_frame=QtWidgets.QFrame.NoFrame,
    box=QtWidgets.QFrame.Box,
    panel=QtWidgets.QFrame.Panel,
    styled_panel=QtWidgets.QFrame.StyledPanel,
    h_line=QtWidgets.QFrame.HLine,
    v_line=QtWidgets.QFrame.VLine,
    win_panel=QtWidgets.QFrame.WinPanel,
)

QtWidgets.QFrame.__bases__ = (widgets.Widget,)


class Frame(QtWidgets.QFrame):
    def set_frame_style(self, style: str):
        """Set frame style.

        Allowed values are "plain", "raised", "sunken"

        Args:
            style: frame style to use

        Raises:
            ValueError: style does not exist
        """
        if style not in SHADOWS:
            raise ValueError("invalid frame style")
        self.setFrameStyle(SHADOWS[style])

    def get_frame_style(self) -> str:
        """Return current frame style.

        Possible values: "plain", "raised", "sunken"

        Returns:
            frame style
        """
        return SHADOWS.inv[self.frameStyle()]



# -*- coding: utf-8 -*-

from prettyqt import gui


class NotEmptyValidator(gui.Validator):
    def __getstate__(self):
        return dict()

    def __setstate__(self, state):
        self.__init__()

    def validate(self, text: str, pos: int = 0) -> tuple:
        if text == "":
            return (self.Intermediate, text, pos)
        return (self.Acceptable, text, pos)


if __name__ == "__main__":
    from prettyqt import widgets

    val = NotEmptyValidator()
    app = widgets.app()
    widget = widgets.LineEdit("This is a test")
    widget.setValidator(val)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Optional, Mapping, Iterable, Union, Dict, Any

from qtpy import QtWidgets

from prettyqt import core, gui, widgets


class SelectionWidget(widgets.GroupBox):
    value_changed = core.Signal(object)

    def __init__(
        self,
        label: str = "",
        layout: str = "horizontal",
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(title=label, parent=parent)
        self.box = widgets.BoxLayout(layout)
        self.widget_custom: Optional[widgets.Widget] = None
        self.rb_other = widgets.RadioButton()
        self.buttons: Dict[widgets.RadioButton, Any] = dict()
        self.set_layout(self.box)

    def __iter__(self):
        return iter(self.buttons.items())

    def add_items(self, items: Union[Iterable, Mapping]):
        if isinstance(items, Mapping):
            for k, v in items.items():
                self.add(v, k)
        else:
            for i in items:
                if isinstance(i, (tuple, list)):
                    self.add(*i)
                else:
                    self.add(i)

    def select_radio_by_data(self, value):
        for rb, data in self.buttons.items():
            if data == value:
                rb.setChecked(True)
                break

    def add(self, title: str, data=None):
        rb = widgets.RadioButton(title)
        rb.toggled.connect(self.update_choice)
        self.buttons[rb] = data
        if len(self.buttons) == 1:
            with rb.block_signals():
                rb.set_value(True)
        self.box += rb

    def add_tooltip_icon(self, text: str):
        label = widgets.Label(text)
        label.setToolTip(text)
        icon = gui.icon.get_icon("mdi.help-circle-outline")
        pixmap = icon.pixmap(core.Size(20, 20))
        label.setPixmap(pixmap)
        self.box += label

    def add_custom(
        self,
        label: str = "Other",
        typ: str = "string",
        default: Union[None, float, str] = None,
        regex: Optional[str] = None,
    ):
        if typ == "string":
            self.widget_custom = widgets.LineEdit()
        elif typ == "int":
            self.widget_custom = widgets.SpinBox()
        elif typ == "float":
            self.widget_custom = widgets.DoubleSpinBox()
        else:
            raise ValueError(typ)
        # TODO: Enable this or add BAR radio and option.
        self.widget_custom.set_disabled()  # type: ignore
        if default is not None:
            self.widget_custom.set_value(default)  # type: ignore
        self.rb_other.setText(label)
        self.rb_other.toggled.connect(self.widget_custom.set_enabled)  # type: ignore
        self.widget_custom.value_changed.connect(  # type: ignore
            lambda: self.update_choice(True)
        )
        if regex and typ == "string":
            self.widget_custom.set_regex_validator(regex)  # type: ignore
        layout = widgets.BoxLayout("horizontal")
        layout += self.rb_other
        layout += self.widget_custom
        self.box += layout

    def current_choice(self):
        for k, v in self.buttons.items():
            if k.isChecked():
                return v
        if self.rb_other.isChecked() and self.widget_custom is not None:
            return self.widget_custom.get_value()
        return

    @core.Slot(bool)
    def update_choice(self, checked):
        if not checked:
            return None
        choice = self.current_choice()
        if choice is not None:
            self.value_changed.emit(choice)

    def set_value(self, value):
        self.select_radio_by_data(value)

    def get_value(self):
        return self.current_choice()


if __name__ == "__main__":
    app = widgets.app()
    widget = SelectionWidget(layout="horizontal")
    items = {";": "Semicolon", "\t": "Tab", ",": "Comma"}
    widget.add_items(items)
    widget.add_custom(label="custom", typ="float")
    widget.show()
    app.exec_()
    print(widget.get_value())



# -*- coding: utf-8 -*-

import pathlib

from qtpy import QtCore


class Dir(QtCore.QDir):
    def __str__(self):
        return self.absolutePath()

    def to_path(self) -> pathlib.Path:
        return pathlib.Path(self.absolutePath())



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QWizard.__bases__ = (widgets.BaseDialog,)


class Wizard(QtWidgets.QWizard):
    def add_widget_as_page(self, widget: QtWidgets.QWidget) -> None:
        page = widgets.WizardPage(self)
        layout = widgets.BoxLayout("vertical", self)
        layout += widget
        page.set_layout(layout)


if __name__ == "__main__":
    app = widgets.app()
    dlg = Wizard()
    dlg.add_widget_as_page(widgets.RadioButton("test"))
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-

import pathlib
from typing import Optional, Union, List, Dict

from qtpy import QtWidgets

from prettyqt import core, widgets


class FileChooserButton(widgets.Widget):

    value_changed = core.Signal(pathlib.Path)

    def __init__(
        self,
        extensions: Optional[Dict[str, List[str]]] = None,
        mode: str = "save",
        file_mode: str = "existing_files",
        root: Union[None, str, pathlib.Path] = None,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        """Initialize FileChooserButton.

        Args:
            extensions: dict allowed extensions
                        form: "'name': ['.ext1', '.ext2']"
            mode: Accept mode ("save" or "load")
            file_mode: File mode ("existing_files", "existing_file", "any_file",
                                  or "directory")
            root: Root path
            parent: parent widget
        """
        super().__init__(parent)
        self.path: Optional[pathlib.Path] = None
        self.extensions = extensions
        self.mode = mode
        self.file_mode = file_mode
        self.root = root
        layout = widgets.BoxLayout("horizontal", self)
        layout.set_margin(0)
        self.lineedit = widgets.LineEdit()
        self.lineedit.set_read_only()
        layout += self.lineedit
        action = widgets.Action()
        if self.file_mode == "directory":
            action.set_icon("mdi.folder-outline")
        else:
            action.set_icon("mdi.file-outline")
        action.triggered.connect(self.open_file)

        self.button = widgets.ToolButton()
        self.button.setDefaultAction(action)
        layout += self.button

    def serialize_fields(self):
        return dict(path=self.path, extensions=self.extensions, enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__(state["extensions"])
        self.set_path(state["path"])
        self.set_enabled(state.get("enabled", True))

    @core.Slot()
    def open_file(self):
        dialog = widgets.FileDialog(
            parent=self,
            path_id="file_path",
            mode=self.mode,
            path=self.root,
            file_mode=self.file_mode,
        )
        if self.extensions:
            dialog.set_extension_filter(self.extensions)
        if not dialog.choose():
            return None
        self.set_path(dialog.selected_file())
        self.value_changed.emit(self.path)

    def set_path(self, path: Union[str, pathlib.Path, None]):
        if isinstance(path, str):
            self.path = pathlib.Path(path)
        else:
            self.path = path
        self.lineedit.set_text(str(path) if path is not None else "")

    def get_value(self) -> Optional[pathlib.Path]:
        return self.path

    def set_value(self, value: Union[str, pathlib.Path]):
        self.set_path(value)


if __name__ == "__main__":
    app = widgets.app()
    btn = FileChooserButton()
    btn.show()
    btn.value_changed.connect(print)
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets


class PopupInfo(widgets.Dialog):
    """Dialog overlay to show some info to user."""

    def __init__(
        self, parent: Optional[QtWidgets.QWidget] = None, text: Optional[str] = None
    ):
        super().__init__(parent=parent)
        self.timer = core.Timer.single_shot(callback=self.close)
        self.label = widgets.Label()
        self.setWindowFlags(
            self.windowFlags()
            | QtCore.Qt.Tool
            | QtCore.Qt.WindowStaysOnTopHint
            | QtCore.Qt.FramelessWindowHint
        )
        layout = widgets.BoxLayout("vertical")
        layout.set_margin(20)
        self.set_layout(layout)
        self.set_background_color("black")
        self.label.set_color("white")
        layout += self.label
        # signals.signals.popup_info.connect(self.popup)

    def show(self, *args, **kwargs):
        self.hide()
        screen_geo = gui.GuiApplication.primaryScreen().geometry()
        size = self.label.sizeHint()
        x = (screen_geo.width() - size.width()) / 2
        y = (screen_geo.height() - size.height()) / 2
        self.move(x, y - 200)
        super().show(*args, **kwargs)
        self.timer.start(2500)

    def show_popup(self, text: str):
        self.label.setText(text)
        self.show()


if __name__ == "__main__":
    app = widgets.app()
    widget = PopupInfo()
    widget.show_popup("test")
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Dict

from prettyqt import constants
from qtpy import QtCore


class SelectionMixin(object):

    CHECKSTATE: Dict = {}  # column: identifier
    dataChanged: QtCore.Signal
    DATA_ROLE: int

    def __init__(self):
        super().__init__()
        self.selected = dict()

    def setData(self, index: QtCore.QModelIndex, value, role):
        if not index.isValid():
            return False
        elif role == constants.CHECKSTATE_ROLE:
            name = self._get_selection_id(index)
            self.selected[name] = not self.selected[name]
            self.dataChanged.emit(index, index)
            return True
        return super().setData(index, value, role)  # type: ignore

    def data(self, index: QtCore.QModelIndex, role=constants.DISPLAY_ROLE):
        if not index.isValid():
            return False
        if role == constants.CHECKSTATE_ROLE:
            if index.column() == 0:
                name = self._get_selection_id(index)
                selected = self.selected.get(name, False)
                if name not in self.selected:
                    self.selected[name] = selected
                return selected
        return super().data(index, role)  # type: ignore

    def flags(self, index: QtCore.QModelIndex):
        flags = super().flags(index)  # type: ignore
        if index.column() in self.CHECKSTATE:
            return flags | constants.IS_CHECKABLE
        return flags

    def _get_selection_id(self, index: QtCore.QModelIndex):
        item = index.data(self.DATA_ROLE)
        id_fn = self.CHECKSTATE.get(index.column())
        if id_fn:
            return id_fn(item)



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtGui


class PolygonF(QtGui.QPolygonF):
    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def __getstate__(self):
        ba = QtCore.QByteArray()
        stream = QtCore.QDataStream(ba, QtCore.QIODevice.WriteOnly)
        stream << self
        return ba

    def __setstate__(self, ba):
        stream = QtCore.QDataStream(ba, QtCore.QIODevice.ReadOnly)
        stream >> self

    @classmethod
    def from_xy(cls, xdata, ydata):
        import numpy as np

        size = len(xdata)
        polyline = cls(size)
        pointer = polyline.data()
        dtype, tinfo = np.float, np.finfo  # integers: = np.int, np.iinfo
        pointer.setsize(2 * polyline.size() * tinfo(dtype).dtype.itemsize)
        memory = np.frombuffer(pointer, dtype)
        memory[: (size - 1) * 2 + 1 : 2] = xdata
        memory[1 : (size - 1) * 2 + 2 : 2] = ydata
        return polyline


if __name__ == "__main__":
    from prettyqt import core

    poly = PolygonF((core.Point(1, 1), core.Point(2, 2)))



# -*- coding: utf-8 -*-

from typing import Callable

from qtpy import QtCore

from prettyqt import core


QtCore.QCoreApplication.__bases__ = (core.Object,)


class CoreApplication(QtCore.QCoreApplication):
    @classmethod
    def call_on_exit(cls, func: Callable):
        cls.instance().aboutToQuit.connect(func)



# -*- coding: utf-8 -*-

from typing import Optional, List

from qtpy import QtWidgets

from prettyqt import gui, widgets


QtWidgets.QInputDialog.__bases__ = (widgets.BaseDialog,)


class InputDialog(QtWidgets.QInputDialog):
    @classmethod
    def get_int(
        cls,
        title: Optional[str] = None,
        label: Optional[str] = None,
        icon: gui.icon.IconType = None,
    ) -> Optional[int]:
        par = widgets.Dialog()
        par.set_icon(icon)
        v = cls.getInt(par, title, label)
        return v[0] if v[1] else None

    @classmethod
    def get_float(
        cls,
        title: Optional[str] = None,
        label: Optional[str] = None,
        icon: gui.icon.IconType = None,
    ) -> Optional[float]:
        par = widgets.Dialog()
        par.set_icon(icon)
        v = cls.getDouble(par, title, label)
        return v[0] if v[1] else None

    @classmethod
    def get_text(
        cls,
        title: Optional[str] = None,
        label: Optional[str] = None,
        icon: gui.icon.IconType = None,
        default_value: str = "",
    ) -> Optional[str]:
        par = widgets.Dialog()
        par.set_icon(icon)
        v = cls.getText(par, title, label, text=default_value)
        return v[0] if v[1] else None

    @classmethod
    def get_item(
        cls,
        items: List[str],
        title: Optional[str] = None,
        label: Optional[str] = None,
        icon: gui.icon.IconType = None,
        editable: bool = False,
    ) -> Optional[str]:
        par = widgets.Dialog()
        par.set_icon(icon)
        v = cls.getItem(par, title, label, items, editable=editable)
        return v[0] if v[1] else None


if __name__ == "__main__":
    app = widgets.app()
    result = InputDialog.get_text("a", "b", icon="mdi.timer")
    print(result)
    app.exec_()



# -*- coding: utf-8 -*-

import pathlib
from typing import Optional
import logging

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import colors

logger = logging.getLogger(__name__)

QtWidgets.QApplication.__bases__ = (gui.GuiApplication,)


class Application(QtWidgets.QApplication):
    def set_icon(self, icon: gui.icon.IconType):
        """Set the default window icon.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon, color=colors.WINDOW_ICON_COLOR)
        self.setWindowIcon(icon)

    def load_language_file(self, path: pathlib.Path):
        translator = core.Translator(self)
        translator.load(str(path))
        self.installTranslator(translator)

    def set_metadata(
        self,
        app_name: Optional[str] = None,
        app_version: Optional[str] = None,
        org_name: Optional[str] = None,
        org_domain: Optional[str] = None,
    ):
        if app_name:
            self.setApplicationName(app_name)
        if app_version:
            self.setApplicationVersion(app_name)
        if org_name:
            self.setOrganizationName(org_name)
        if org_domain:
            self.setOrganizationDomain(org_domain)

    def about_popup(self, title="About"):
        text = (
            f"{self.applicationName()}\n\n"
            f"{self.organizationName()}\n"
            f"{self.applicationVersion()}\n"
            f"{self.organizationDomain()}"
        )
        popup = widgets.MessageBox(
            widgets.MessageBox.NoIcon, title, text, buttons=widgets.MessageBox.Ok
        )
        popup.set_icon("mdi.information-outline")
        popup.exec_()

    def main_loop(self):
        return self.exec_()

    @classmethod
    def use_hdpi_bitmaps(cls, state: bool = True):
        cls.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, state)

    @classmethod
    def disable_window_help_button(cls, state: bool = True):
        cls.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton, state)

    @classmethod
    def copy_to_clipboard(cls, text: str):
        """Sets clipboard to supplied text."""
        cb = cls.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(text, mode=cb.Clipboard)

    @classmethod
    def get_mainwindow(cls) -> Optional[QtWidgets.QMainWindow]:
        widget_list = cls.instance().topLevelWidgets()
        for widget in widget_list:
            if isinstance(widget, QtWidgets.QMainWindow):
                return widget
        return None

    @classmethod
    def get_widget(cls, name: str) -> Optional[QtWidgets.QWidget]:
        mw = cls.get_mainwindow()
        if mw is None:
            logger.warning("Trying to get widget from nonexistent mainwindow")
            return None
        return mw.findChild(QtWidgets.QWidget, name)
        # widget_list = cls.instance().allWidgets()
        # for widget in widget_list:
        #     if isinstance(widget, QtWidgets.QWidget) and widget.objectName() == name:
        #         return widget
        # return None



# -*- coding: utf-8 -*-

"""Core module.

Contains QtCore-based classes
"""

# from qtpy.QtCore import Signal

from qtpy.QtCore import Slot, Signal, Property
from qtpy.QtCore import QModelIndex as ModelIndex
from qtpy.QtCore import QPoint as Point
from qtpy.QtCore import QPointF as PointF
from qtpy.QtCore import QSizeF as SizeF
from qtpy.QtCore import QRect as Rect
from qtpy.QtCore import QRectF as RectF

from .object import Object
from .url import Url
from .coreapplication import CoreApplication
from .iodevice import IODevice
from .filedevice import FileDevice
from .file import File
from .buffer import Buffer
from .settings import Settings
from .date import Date
from .datetime import DateTime

from .size import Size

# from .sizef import SizeF
# from .point import Point
# from .pointf import PointF
from .timer import Timer
from .translator import Translator
from .thread import Thread

# from .rect import Rect
# from .rectf import RectF
from .regularexpressionmatch import RegularExpressionMatch
from .regularexpressionmatchiterator import RegularExpressionMatchIterator
from .regexp import RegExp
from .regularexpression import RegularExpression
from .mimedata import MimeData
from .runnable import Runnable

# from .modelindex import ModelIndex
from .threadpool import ThreadPool
from .dir import Dir
from .event import Event
from .itemselectionmodel import ItemSelectionModel
from .diriterator import DirIterator
from .abstractitemmodel import AbstractItemModel
from .abstractproxymodel import AbstractProxyModel
from .abstractlistmodel import AbstractListModel
from .sortfilterproxymodel import SortFilterProxyModel
from .abstracttablemodel import AbstractTableModel
from .standardpaths import StandardPaths

__all__ = [
    "Object",
    "Url",
    "CoreApplication",
    "IODevice",
    "FileDevice",
    "File",
    "Buffer",
    "Settings",
    "Date",
    "DateTime",
    "Size",
    "SizeF",
    "Point",
    "PointF",
    "Timer",
    "Translator",
    "Thread",
    "Rect",
    "RectF",
    "MimeData",
    "Dir",
    "Event",
    "DirIterator",
    "ItemSelectionModel",
    "Slot",
    "Property",
    "RegularExpressionMatch",
    "RegularExpressionMatchIterator",
    "RegExp",
    "RegularExpression",
    "Runnable",
    "ModelIndex",
    "ThreadPool",
    "Signal",
    "AbstractItemModel",
    "AbstractProxyModel",
    "AbstractListModel",
    "SortFilterProxyModel",
    "AbstractTableModel",
    "StandardPaths",
]



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets

from prettyqt import core
from prettyqt.utils import bidict


POLICIES = bidict(
    none=QtWidgets.QActionGroup.ExclusionPolicy(0),
    exclusive=QtWidgets.QActionGroup.ExclusionPolicy.Exclusive,
    exclusive_optional=QtWidgets.QActionGroup.ExclusionPolicy.ExclusiveOptional,
)


QtWidgets.QActionGroup.__bases__ = (core.Object,)


class ActionGroup(QtWidgets.QActionGroup):
    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(parent)

    def __len__(self) -> int:
        return len(self.actions())

    def __getitem__(self, item: int):
        return self.actions()[item]

    def set_exclusion_policy(self, policy: Optional[str]):
        """Set exclusion policy to use.

        Allowed values are "none", "exclusive", "exclusive_optional"

        Args:
            policy: exclusion policy to use

        Raises:
            ValueError: exclusion policy does not exist
        """
        if policy is None:
            policy = "none"
        if policy not in POLICIES:
            raise ValueError(f"Invalid exclusion policy. Valid values: {POLICIES.keys()}")
        self.setExclusionPolicy(POLICIES[policy])

    def get_exclusion_policy(self) -> str:
        """Return current exclusion policy.

        Possible values: "none", "exclusive", "exclusive_optional"

        Returns:
            exclusion policy
        """
        return POLICIES.inv[self.exclusionPolicy()]


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    action = ActionGroup()
    act = widgets.Action()
    action.addAction(act)
    print(act in action)
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets
from typing import Optional


QtWidgets.QWidgetAction.__bases__ = (widgets.Action,)


class WidgetAction(QtWidgets.QWidgetAction):
    def __init__(self, parent: Optional[QtWidgets.QWidget] = None) -> None:
        super().__init__(parent)

    #     self.set_text(text)
    #     self.set_icon(icon)
    #     self.set_shortcut(shortcut)
    #     self.set_tooltip(tooltip)



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtGui

from prettyqt import gui
from prettyqt.utils import bidict, colors


UNDERLINE_STYLES = bidict(
    none=QtGui.QTextCharFormat.NoUnderline,
    single=QtGui.QTextCharFormat.SingleUnderline,
    dash=QtGui.QTextCharFormat.DashUnderline,
    dot=QtGui.QTextCharFormat.DotLine,
    dashdot=QtGui.QTextCharFormat.DashDotLine,
    dashdotline=QtGui.QTextCharFormat.DashDotDotLine,
    wave=QtGui.QTextCharFormat.WaveUnderline,
    spellcheck=QtGui.QTextCharFormat.SpellCheckUnderline,
)


WEIGHTS = gui.font.WEIGHTS  # type: ignore

STYLE_HINTS = gui.font.STYLE_HINTS  # type: ignore


class TextCharFormat(QtGui.QTextCharFormat):
    def __init__(
        self,
        text_color: Union[colors.ColorType, QtGui.QBrush] = None,
        bold: bool = False,
        italic: bool = False,
    ):
        super().__init__()
        if text_color is not None:
            self.set_foreground_color(text_color)
        if bold:
            self.set_font_weight("bold")
        self.setFontItalic(italic)

    def set_foreground_color(self, color: Union[colors.ColorType, QtGui.QBrush]):
        if not isinstance(color, QtGui.QBrush):
            color = colors.get_color(color)
        self.setForeground(color)

    def set_background_color(self, color: Union[colors.ColorType, QtGui.QBrush]):
        if not isinstance(color, QtGui.QBrush):
            color = colors.get_color(color)
        self.setBackground(color)

    def set_font_weight(self, weight: str):
        """Set the font weight.

        Valid values are "thin", "extra_light", light", "medium", "demi_bold", "bold",
                         "extra_bold", normal", "black"

        Args:
            weight: font weight

        Raises:
            ValueError: invalid font weight
        """
        if weight not in WEIGHTS:
            raise ValueError("Invalid font weight")
        self.setFontWeight(WEIGHTS[weight])

    def get_font_weight(self) -> str:
        """Get current font weight.

        Possible values are "thin", "light", "medium" or "bold"

        Returns:
            current font weight
        """
        return WEIGHTS.inv[self.fontWeight()]

    def set_underline_style(self, style: str):
        """Set the underline style.

        Valid values are "none", "single", "dash", "dot", "dashdot", "dashdotline",
        "wave", "spellcheck"

        Args:
            style: underline style

        Raises:
            ValueError: invalid underline style
        """
        if style not in UNDERLINE_STYLES:
            raise ValueError("Invalid underline style")
        self.setUnderlineStyle(UNDERLINE_STYLES[style])

    def get_underline_style(self) -> str:
        """Get current underline style.

        Possible values are "none", "single", "dash", "dot", "dashdot", "dashdotline",
        "wave", "spellcheck"

        Returns:
            current underline style
        """
        return UNDERLINE_STYLES.inv[self.underlineStyle()]

    def set_font_style_hint(self, hint: str):
        """Set the font style hint.

        Valid values are "any", "sans_serif", "serif", "typewriter", "decorative",
        "monospace", "fantasy", "cursive", "system"

        Args:
            hint: font style hint

        Raises:
            ValueError: invalid font style hint
        """
        if hint not in STYLE_HINTS:
            raise ValueError("Invalid font style hint")
        self.setFontStyleHint(STYLE_HINTS[hint])

    def select_full_width(self, value: bool = True):
        self.setProperty(QtGui.QTextFormat.FullWidthSelection, value)



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtGui

from prettyqt import core, gui


class StandardItem(QtGui.QStandardItem):
    def __repr__(self):
        return f"StandardItem({self.icon()}, {self.text()!r})"

    def __getstate__(self):
        return dict(
            text=self.text(),
            tooltip=self.toolTip(),
            statustip=self.statusTip(),
            icon=gui.Icon(self.icon()) if not self.icon().isNull() else None,
            data=self.data(),
        )

    def __setstate__(self, state):
        self.__init__()
        self.setText(state.get("text", ""))
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))
        self.setData(state["data"])
        self.set_icon(state["icon"])

    def clone(self):
        item = self.__class__()
        ba = QtCore.QByteArray()
        ds = QtCore.QDataStream(ba, core.IODevice.WriteOnly)
        ds << self
        ds = QtCore.QDataStream(ba)
        ds >> item
        assert type(item) == StandardItem
        return item

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import core


QtWidgets.QAbstractItemDelegate.__bases__ = (core.Object,)


class AbstractItemDelegate(QtWidgets.QAbstractItemDelegate):
    pass



# -*- coding: utf-8 -*-

"""Webenginewidgets module.

contains QWebEngineView-based classes
"""

import logging

logger = logging.getLogger(__name__)

try:
    from .webengineview import WebEngineView
    from .webenginepage import WebEnginePage

    __all__ = ["WebEngineView", "WebEnginePage"]
except ModuleNotFoundError:
    logger.warning("Tried to import WebEngineView but not installed.")



# -*- coding: utf-8 -*-

from typing import List, Optional

from qtpy import QtCore
from prettyqt import gui


class CompositeValidator(gui.Validator):
    def __init__(
        self,
        validators: List[gui.Validator] = None,
        parent: Optional[QtCore.QObject] = None,
    ):
        super().__init__(parent)
        self.validators = validators if validators is not None else []

    def __repr__(self):
        return f"CompositeValidator({self.validators})"

    def __getstate__(self):
        return dict(validators=self.validators)

    def __setstate__(self, state):
        self.__init__()
        self.validators = state.get("validators", [])

    def validate(self, text: str, pos: int = 0) -> tuple:
        vals = [v.validate(text, pos)[0] for v in self.validators]
        return (min(vals), text, pos)


if __name__ == "__main__":
    from prettyqt import custom_validators, widgets

    val1 = custom_validators.NotEmptyValidator()
    val2 = custom_validators.PathValidator()
    val = CompositeValidator([val1, val2])
    app = widgets.app()
    widget = widgets.LineEdit("This is a test")
    widget.setValidator(val)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

import pathlib
from typing import Union, Optional, Dict, Tuple


import qtawesome as qta
from qtpy import QtCore, QtGui

from prettyqt import core, gui


IconType = Union[QtGui.QIcon, str, pathlib.Path, None]

key_type = Tuple[Optional[str], Optional[str], bool]
icon_cache: Dict[key_type, QtGui.QIcon] = dict()


def get_icon(icon: IconType, color: Optional[str] = None, as_qicon: bool = False):
    """Get icon with given color.

    Qtawesome already caches icons, but since we construct our own subclassed icon,
    we cache, too.
    """
    if isinstance(icon, QtGui.QIcon):
        return icon if as_qicon else Icon(icon)
    if isinstance(icon, pathlib.Path):
        icon = str(icon)
    if (icon, color, as_qicon) in icon_cache:
        return icon_cache[(icon, color, as_qicon)]
    if isinstance(icon, str) and icon.startswith("mdi."):
        if color is not None:
            new = qta.icon(icon, color=color)
        else:
            new = qta.icon(icon)
    else:
        new = QtGui.QIcon(icon)
    icon = new if as_qicon else Icon(new)
    icon_cache[(icon, color, as_qicon)] = icon
    return icon


def set_defaults(*args, **kwargs):
    qta.set_defaults(*args, **kwargs)


class Icon(QtGui.QIcon):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    def __bool__(self):
        return not bool(self.isNull())

    def __getstate__(self):
        ba = QtCore.QByteArray()
        stream = QtCore.QDataStream(ba, QtCore.QIODevice.WriteOnly)
        pixmap = self.pixmap(QtCore.QSize(256, 256))
        stream << pixmap
        return ba

    def __setstate__(self, ba):
        stream = QtCore.QDataStream(ba, QtCore.QIODevice.ReadOnly)
        px = QtGui.QPixmap()
        stream >> px
        super().__init__(px)

    @classmethod
    def for_color(cls, color_str: str) -> "Icon":
        color = gui.Color.from_text(color_str)
        if color.isValid():
            bitmap = gui.Pixmap(16, 16)
            bitmap.fill(color)
            icon = cls(bitmap)
        else:
            icon = cls(qta.icon("mdi.card-outline"))
        return icon

    @classmethod
    def from_image(cls, image: QtGui.QImage):
        return cls(gui.Pixmap.fromImage(image))

    def get_pixmap(self, size: int) -> QtGui.QPixmap:
        size = core.Size(size, size)
        return self.pixmap(size)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    icon = Icon.for_color("green")



# -*- coding: utf-8 -*-

from qtpy import QtMultimedia

from prettyqt import core


class MediaContent(QtMultimedia.QMediaContent):
    def get_url(self):
        return core.Url(self.canonicalUrl())



# -*- coding: utf-8 -*-

from typing import List
import warnings
import functools
import sys


def string_to_num_array(array: str) -> List[float]:
    floats = [float(i) for i in array.split(",")]
    return [int(i) if i.is_integer() else i for i in floats]


def deprecated(func):
    """A decorator which can be used to mark functions as deprecated.

    It will result in a warning being emitted
    when the function is used.
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter("always", DeprecationWarning)  # turn off filter
        warnings.warn(
            "Call to deprecated function {}.".format(func.__name__),
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter("default", DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func


def is_dark_mode() -> bool:
    if sys.platform.startswith("win"):
        from prettyqt import core

        p = (
            "HKEY_CURRENT_USER\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\"
            "Themes\\Personalize"
        )
        s = core.Settings(p, core.Settings.NativeFormat)
        return s.value("AppsUseLightTheme") == 0
    elif sys.platform == "darwin":
        import darkdetect

        return darkdetect.isDark()
    else:
        return False



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtWidgets

from prettyqt import gui, widgets


QtWidgets.QUndoView.__bases__ = (widgets.ListView,)


class UndoView(QtWidgets.QUndoView):
    def __getitem__(self, index: int):
        return self.stack()[index]

    def set_clean_icon(self, icon: gui.icon.IconType):
        """Set the icon for the clean button.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setCleanIcon(icon)

    def set_value(self, value: Union[QtWidgets.QUndoGroup, QtWidgets.QUndoStack]):
        if isinstance(value, QtWidgets.QUndoGroup):
            self.setGroup(value)
        else:
            self.setStack(value)


if __name__ == "__main__":
    app = widgets.app()
    view = UndoView()
    stack = widgets.UndoStack()
    stack.add_command("test", redo=lambda: print("redo"), undo=lambda: print("undo"))
    view.setStack(stack)
    view.show()
    app.exec_()



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import gui
from prettyqt.utils import bidict


STATES = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)


class ListWidgetItem(QtWidgets.QListWidgetItem):
    def __repr__(self):
        return f"ListWidgetItem({self.icon()}, {self.text()!r})"

    def __getstate__(self):
        return dict(
            text=self.text(),
            tooltip=self.toolTip(),
            statustip=self.statusTip(),
            checkstate=self.get_checkstate(),
            icon=gui.Icon(self.icon()) if not self.icon().isNull() else None,
            data=self.data(QtCore.Qt.UserRole),
        )

    def __setstate__(self, state):
        self.__init__()
        self.setText(state.get("text", ""))
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))
        self.setData(QtCore.Qt.UserRole, state["data"])
        self.set_icon(state["icon"])
        self.set_checkstate(state["checkstate"])

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)

    def set_checkstate(self, state: str):
        """Set checkstate of the checkbox.

        valid values are: unchecked, partial, checked

        Args:
            state: checkstate to use

        Raises:
            ValueError: invalid checkstate
        """
        if state not in STATES:
            raise ValueError("Invalid checkstate.")
        self.setCheckState(STATES[state])

    def get_checkstate(self) -> str:
        """Return checkstate.

        possible values are "unchecked", "partial", "checked"

        Returns:
            checkstate
        """
        return STATES.inv[self.checkState()]



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import gui, widgets
from prettyqt.utils import bidict


POPUP_MODES = bidict(
    delayed=QtWidgets.QToolButton.DelayedPopup,
    menu_button=QtWidgets.QToolButton.MenuButtonPopup,
    instant=QtWidgets.QToolButton.InstantPopup,
)

ARROW_TYPES = bidict(
    none=QtCore.Qt.NoArrow,
    up=QtCore.Qt.UpArrow,
    down=QtCore.Qt.DownArrow,
    left=QtCore.Qt.LeftArrow,
    right=QtCore.Qt.RightArrow,
)

STYLES = bidict(
    icon=QtCore.Qt.ToolButtonIconOnly,
    text=QtCore.Qt.ToolButtonTextOnly,
    text_beside_icon=QtCore.Qt.ToolButtonTextBesideIcon,
    text_below_icon=QtCore.Qt.ToolButtonTextUnderIcon,
)

QtWidgets.QToolButton.__bases__ = (widgets.AbstractButton,)


class ToolButton(QtWidgets.QToolButton):
    def __getitem__(self, item):
        menu = self.menu()
        return menu[item]

    @classmethod
    def for_menu(cls, menu: QtWidgets.QMenu, icon: gui.icon.IconType = None):
        btn = cls()
        btn.setMenu(menu)
        btn.set_popup_mode("instant")
        btn.set_icon(icon)
        return btn

    def set_default_action(self, action):
        self.setDefaultAction(action)

    def set_popup_mode(self, mode: str):
        """Set the popup mode of the toolbutton.

        valid values are: "delayed", "menu_button", "instant"

        Args:
            mode: popup mode to use

        Raises:
            ValueError: invalid popup mode
        """
        if mode not in POPUP_MODES:
            raise ValueError("Invalid mode.")
        self.setPopupMode(POPUP_MODES[mode])

    def get_popup_mode(self) -> str:
        """Return popup mode.

        possible values are "delayed", "menu_button", "instant"

        Returns:
            popup mode
        """
        return POPUP_MODES.inv[self.popupMode()]

    def set_arrow_type(self, mode: str):
        """Set the arrow type of the toolbutton.

        valid values are: "none", "up", "down", "left", "right"

        Args:
            mode: arrow type to use

        Raises:
            ValueError: invalid arrow type
        """
        if mode not in ARROW_TYPES:
            raise ValueError("Invalid arrow type.")
        self.setArrowType(ARROW_TYPES[mode])

    def get_arrow_type(self) -> str:
        """Return arrow type.

        possible values are "none", "up", "down", "left", "right"

        Returns:
            arrow type
        """
        return ARROW_TYPES.inv[self.arrowType()]

    def set_style(self, style: str):
        if style not in STYLES:
            raise ValueError("Invalid style.")
        self.setToolButtonStyle(STYLES[style])


if __name__ == "__main__":
    w = ToolButton()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets

from prettyqt import widgets


class OptionalWidget(widgets.GroupBox):
    def __init__(
        self,
        widget: QtWidgets.QWidget,
        title: str = "",
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(checkable=True, title=title)
        self.set_layout("vertical")
        self.box.add(widget)
        self.widget = widget
        self.toggled.connect(self.widget.setEnabled)

    def __getattr__(self, value):
        return self.widget.__getattribute__(value)

    @property
    def enabled(self) -> bool:
        return self.isChecked()

    @enabled.setter
    def enabled(self, state: bool):
        self.setChecked(state)

    def get_value(self):
        if self.isChecked():
            return self.widget.get_value()
        return None


if __name__ == "__main__":
    app = widgets.app()
    img = widgets.RadioButton("test")
    widget = OptionalWidget(img, "Test")
    widget.show()
    app.exec_()
    print(widget.enabled)



# -*- coding: utf-8 -*-

from typing import Union

import pathlib

from qtpy import QtCore


class Url(QtCore.QUrl):
    def __init__(self, path: Union[QtCore.QUrl, str, pathlib.Path]):

        super().__init__(str(path) if not isinstance(path, QtCore.QUrl) else path)
        if isinstance(path, pathlib.Path):
            self.setScheme("file")

    # def __str__(self):
    #     return self.absolutePath()

    def __repr__(self):
        return f"core.Url('{self.toString(self.PreferLocalFile)}')"

    def __str__(self):
        return self.toString(self.PreferLocalFile)

    def to_path(self) -> pathlib.Path:
        """Get pathlib object from the URL.

        Returns:
            Path
        """
        return pathlib.Path(str(self))

    def is_local_file(self) -> bool:
        return self.isLocalFile()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import gui, widgets
from prettyqt.utils import bidict


CORRECTION_MODES = bidict(
    to_previous=QtWidgets.QSpinBox.CorrectToPreviousValue,
    to_nearest=QtWidgets.QSpinBox.CorrectToNearestValue,
)

SYMBOLS = bidict(
    up_down=QtWidgets.QSpinBox.UpDownArrows,
    plus_minus=QtWidgets.QSpinBox.PlusMinus,
    none=QtWidgets.QSpinBox.NoButtons,
)

STEP_TYPES = bidict(
    default=QtWidgets.QSpinBox.DefaultStepType,
    adaptive=QtWidgets.QSpinBox.AdaptiveDecimalStepType,
)


QtWidgets.QAbstractSpinBox.__bases__ = (widgets.Widget,)


class AbstractSpinBox(QtWidgets.QAbstractSpinBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLineEdit(widgets.LineEdit())
        self.setGroupSeparatorShown(True)

    def serialize_fields(self):
        return dict(
            button_symbols=self.get_button_symbols(),
            correction_mode=self.get_correction_mode(),
        )

    def is_valid(self) -> bool:
        return self.hasAcceptableInput()

    def set_validator(self, validator: gui.Validator):
        self.lineEdit().setValidator(validator)

    def get_button_symbols(self) -> str:
        """Return button symbol type.

        possible values are "none", "up_down", "plus_minus"

        Returns:
            button symbol type
        """
        return SYMBOLS.inv[self.buttonSymbols()]

    def set_button_symbols(self, mode: str):
        """Set button symbol type.

        possible values are "none", "up_down", "plus_minus"

        Args:
            mode: button symbol type to use

        Raises:
            ValueError: invalid button symbol type
        """
        if mode not in SYMBOLS:
            raise ValueError("Invalid button symbol type.")
        self.setButtonSymbols(SYMBOLS[mode])

    def set_correction_mode(self, mode: str):
        """Set correction mode.

        possible values are "to_previous", "to_nearest"

        Args:
            mode: correction mode to use

        Raises:
            ValueError: invalid correction mode
        """
        if mode not in CORRECTION_MODES:
            raise ValueError("Invalid correction mode.")
        self.setCorrectionMode(CORRECTION_MODES[mode])

    def get_correction_mode(self) -> str:
        """Return correction mode.

        possible values are "to_previous", "to_nearest"

        Returns:
            correction mode
        """
        return CORRECTION_MODES.inv[self.correctionMode()]

    def set_step_type(self, mode: str):
        """Set step type.

        possible values are "default", "adaptive"

        Args:
            mode: step type to use

        Raises:
            ValueError: invalid step type
        """
        if mode not in STEP_TYPES:
            raise ValueError("Invalid step type.")
        self.setStepType(STEP_TYPES[mode])

    def get_step_type(self) -> str:
        """Return step type.

        possible values are "default", "adaptive"

        Returns:
            step type
        """
        return STEP_TYPES.inv[self.stepType()]

    def set_special_value(self, value: str):
        self.setSpecialValueText(value)

    def get_value(self) -> int:
        return self.value()

    def set_value(self, value: int):
        self.setValue(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = AbstractSpinBox()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

import orjson as json
from qtpy import QtCore

from prettyqt import core


QtCore.QMimeData.__bases__ = (core.Object,)
OPTS = json.OPT_NAIVE_UTC | json.OPT_SERIALIZE_NUMPY


class MimeData(QtCore.QMimeData):
    def set_data(self, mime_type: str, data: str):
        self.setData(mime_type, QtCore.QByteArray(data.encode()))

    def set_json_data(self, mime_type: str, data):
        self.setData(mime_type, QtCore.QByteArray(json.dumps(data, option=OPTS)))

    def get_data(self, mime_type: str) -> str:
        return bytes(self.data(mime_type)).decode()

    def get_json_data(self, mime_type: str):
        data = self.data(mime_type)
        return json.loads(bytes(data))



# -*- coding: utf-8 -*-

"""Gui module.

Contains QtGui-based classes
"""

from qtpy.QtGui import QKeyEvent as KeyEvent
from qtpy.QtGui import QMouseEvent as MouseEvent
from qtpy.QtGui import QWheelEvent as WheelEvent
from qtpy.QtGui import QActionEvent as ActionEvent
from qtpy.QtGui import QShowEvent as ShowEvent
from qtpy.QtGui import QContextMenuEvent as ContextMenuEvent

from .icon import Icon
from .guiapplication import GuiApplication
from .validator import Validator
from .regexpvalidator import RegExpValidator

try:
    from .regularexpressionvalidator import RegularExpressionValidator
except AttributeError:
    from .regularexpressionvalidator_pyside import (  # type: ignore
        RegularExpressionValidator,
    )
from .intvalidator import IntValidator
from .doublevalidator import DoubleValidator
from .brush import Brush
from .textblockuserdata import TextBlockUserData
from .color import Color
from .font import Font
from .fontmetrics import FontMetrics
from .fontdatabase import FontDatabase
from .region import Region
from .paintdevice import PaintDevice
from .pagedpaintdevice import PagedPaintDevice
from .image import Image
from .pen import Pen
from .picture import Picture
from .pixmap import Pixmap
from .painter import Painter
from .palette import Palette
from .cursor import Cursor
from .polygonf import PolygonF
from .standarditem import StandardItem
from .standarditemmodel import StandardItemModel
from .textcharformat import TextCharFormat
from .textcursor import TextCursor
from .syntaxhighlighter import SyntaxHighlighter
from .pdfwriter import PdfWriter
from .keysequence import KeySequence
from .desktopservices import DesktopServices


__all__ = [
    "KeyEvent",
    "MouseEvent",
    "WheelEvent",
    "ActionEvent",
    "ShowEvent",
    "ContextMenuEvent",
    "GuiApplication",
    "Validator",
    "RegExpValidator",
    "RegularExpressionValidator",
    "IntValidator",
    "DoubleValidator",
    "Brush",
    "TextBlockUserData",
    "Color",
    "Font",
    "FontMetrics",
    "FontDatabase",
    "Region",
    "PaintDevice",
    "PagedPaintDevice",
    "Icon",
    "Image",
    "Pen",
    "Picture",
    "Pixmap",
    "Painter",
    "Palette",
    "Cursor",
    "PolygonF",
    "StandardItem",
    "StandardItemModel",
    "TextCharFormat",
    "TextCursor",
    "SyntaxHighlighter",
    "PdfWriter",
    "KeySequence",
    "DesktopServices",
]



# -*- coding: utf-8 -*-

"""Multimediawidgets module."""

from .videowidgetcontrol import VideoWidgetControl
from .videowidget import VideoWidget

__all__ = [
    "VideoWidgetControl",
    "VideoWidget",
]



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtGui

from prettyqt.utils import bidict


SHAPES = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
)


class Cursor(QtGui.QCursor):
    def set_shape(self, shape: str):
        """Set cursor shape.

        Allowed values are "arrow", "uparrow", "cross", "wait", "caret"

        Args:
            shape: shape to use

        Raises:
            ValueError: shape does not exist
        """
        if shape not in SHAPES:
            raise ValueError(f"Invalid shape type '{shape}.")
        self.setShape(SHAPES[shape])

    def get_shape(self) -> str:
        """Return current cursor shape.

        Possible values: "arrow", "uparrow", "cross", "wait", "caret"

        Returns:
            cursor shape
        """
        return SHAPES.inv[self.shape()]



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui


QtGui.QPagedPaintDevice.__bases__ = (gui.PaintDevice,)


class PagedPaintDevice(QtGui.QPagedPaintDevice):
    pass


if __name__ == "__main__":
    device = PagedPaintDevice()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui


QtGui.QImage.__bases__ = (gui.PaintDevice,)


class Image(QtGui.QImage):
    pass


if __name__ == "__main__":
    image = Image()



# -*- coding: utf-8 -*-

import pathlib
from typing import Callable, Union

from qtpy import QtWebEngineWidgets

from prettyqt import core, widgets


QtWebEngineWidgets.QWebEngineView.__bases__ = (widgets.Widget,)


class WebEngineView(QtWebEngineWidgets.QWebEngineView):
    def set_url(self, url: Union[str, pathlib.Path]):
        """Set the url of the WebEngineView.

        Clears the view and loads the URL.

        Args:
            url: URL to set
        """
        if isinstance(url, pathlib.Path):
            url = core.Url.fromLocalFile(str(url))
        elif isinstance(url, str):
            url = core.Url(url)
        self.setUrl(url)

    def load_url(self, url: Union[str, pathlib.Path]):
        """Load the URL.

        Loads the specified url and displays it.

        Note: The view remains the same until enough data has arrived
        to display the new URL.

        Args:
            url: URL to load
        """
        if isinstance(url, pathlib.Path):
            url = core.Url.fromLocalFile(str(url))
        elif isinstance(url, str):
            url = core.Url(url)
        self.load(url)

    def set_zoom(self, zoom: float):
        """Set the zoom factor for the view.

        Valid values are within the range from 0.25 to 5.0. The default factor is 1.0.

        Args:
            zoom: Zoom factor
        """
        self.setZoomFactor(zoom)

    def find_text(
        self,
        string: str,
        backward: bool = False,
        case_sensitive: bool = False,
        callback: Callable = None,
    ):
        """Find text in the current page.

        Finds the specified string, subString, in the page, using the given options.
        The findTextFinished() signal is emitted when a string search is completed.

        To clear the search highlight, just pass an empty string.

        The resultCallback must take a boolean parameter.
        It will be called with a value of true if the subString was found;
        otherwise the callback value will be false.

        Warning: It is guaranteed that the callback is always called,
        but it might be done during page destruction. When WebEnginePage is deleted,
        the callback is triggered with an invalid value and it is not safe to use
        the corresponding QWebEnginePage or QWebEngineView instance inside it.

        Args:
            string: string to search for
            backward: search backwards
            case_sensitive: case-sensitive search
            callback: result callback
        """
        if callback is None:

            def do_nothing(x):
                pass

            callback = do_nothing
        flag = QtWebEngineWidgets.QWebEnginePage.FindFlag()
        if case_sensitive:
            flag |= QtWebEngineWidgets.QWebEnginePage.FindCaseSensitively
        if backward:
            flag |= QtWebEngineWidgets.QWebEnginePage.FindBackward
        self.findText(string, flag, callback)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    path = path = "E:\\dev\\datacook\\processanalyzer\\docs\\index.html"
    widget = WebEngineView()
    widget.set_url(path)
    widget.find_text("test", backward=True, case_sensitive=True, callback=None)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Union
from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStatusBar.__bases__ = (widgets.Widget,)


class StatusBar(QtWidgets.QStatusBar):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.progress_bar = widgets.ProgressBar()

    def __add__(self, other: Union[QtWidgets.QAction, QtWidgets.QWidget]) -> "StatusBar":
        if isinstance(other, QtWidgets.QAction):
            self.add_action(other)
            return self
        elif isinstance(other, QtWidgets.QWidget):
            self.addWidget(other)
            return self
        else:
            raise TypeError(other)

    def setup_default_bar(self) -> None:
        # This is simply to show the bar
        self.progress_bar.hide()
        self.progress_bar.setRange(0, 0)
        self.progress_bar.setFixedSize(200, 20)
        self.progress_bar.setTextVisible(False)
        self.addPermanentWidget(self.progress_bar)

    def add_action(self, action: QtWidgets.QAction) -> None:
        self.addAction(action)

    def add_widget(self, widget: QtWidgets.QWidget, permanent: bool = False) -> None:
        if permanent:
            self.addPermanentWidget(widget)
        else:
            self.addWidget(widget)

    def show_message(self, message: str, timeout: int = 0) -> None:
        self.showMessage(message, timeout)


if __name__ == "__main__":
    app = widgets.app()
    dlg = widgets.MainWindow()
    status_bar = StatusBar()
    status_bar.set_color("black")
    label = widgets.Label("test")
    status_bar.addWidget(label)
    status_bar.setup_default_bar()
    dlg.setStatusBar(status_bar)
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-

import contextlib

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import bidict

CURSOR_SHAPES = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
    size_vertical=QtCore.Qt.SizeVerCursor,
    size_horizonal=QtCore.Qt.SizeHorCursor,
    size_topright=QtCore.Qt.SizeBDiagCursor,
    size_topleft=QtCore.Qt.SizeFDiagCursor,
    size_all=QtCore.Qt.SizeAllCursor,
    blank=QtCore.Qt.BlankCursor,
    split_vertical=QtCore.Qt.SplitVCursor,
    split_horizontal=QtCore.Qt.SplitHCursor,
    pointing_hand=QtCore.Qt.PointingHandCursor,
    forbidden=QtCore.Qt.ForbiddenCursor,
    open_hand=QtCore.Qt.OpenHandCursor,
    closed_hand=QtCore.Qt.ClosedHandCursor,
    whats_this=QtCore.Qt.WhatsThisCursor,
    busy=QtCore.Qt.BusyCursor,
    drag_move=QtCore.Qt.DragMoveCursor,
    drag_copy=QtCore.Qt.DragCopyCursor,
    drag_link=QtCore.Qt.DragLinkCursor,
    bitmap=QtCore.Qt.BitmapCursor,
)


QtGui.QGuiApplication.__bases__ = (core.CoreApplication,)


class GuiApplication(QtGui.QGuiApplication):
    @classmethod
    @contextlib.contextmanager
    def override_cursor(cls, cursor: str):
        cls.set_override_cursor(cursor)
        yield cursor
        cls.restore_override_cursor()

    @classmethod
    def set_override_cursor(cls, cursor: str):
        cursor = gui.Cursor(CURSOR_SHAPES[cursor])
        cls.setOverrideCursor(cursor)

    @classmethod
    def restore_override_cursor(cls):
        cls.restoreOverrideCursor()



# -*- coding: utf-8 -*-

"""Constants module."""

from qtpy import QtCore

DISPLAY_ROLE = QtCore.Qt.DisplayRole
USER_ROLE = QtCore.Qt.UserRole
SORT_ROLE = QtCore.Qt.UserRole + 1
NAME_ROLE = QtCore.Qt.UserRole + 2
EDIT_ROLE = QtCore.Qt.EditRole
BACKGROUND_ROLE = QtCore.Qt.BackgroundRole
FOREGROUND_ROLE = QtCore.Qt.ForegroundRole
TOOLTIP_ROLE = QtCore.Qt.ToolTipRole
STATUSTIP_ROLE = QtCore.Qt.StatusTipRole
DECORATION_ROLE = QtCore.Qt.DecorationRole
CHECKSTATE_ROLE = QtCore.Qt.CheckStateRole
ALIGNMENT_ROLE = QtCore.Qt.TextAlignmentRole
FONT_ROLE = QtCore.Qt.FontRole

ALIGN_LEFT = QtCore.Qt.AlignLeft
ALIGN_RIGHT = QtCore.Qt.AlignRight
ALIGN_CENTER = QtCore.Qt.AlignCenter
ALIGN_BOTTOM = QtCore.Qt.AlignBottom
ALIGN_V_CENTER = QtCore.Qt.AlignVCenter

HORIZONTAL = QtCore.Qt.Horizontal
VERTICAL = QtCore.Qt.Vertical

ASCENDING = QtCore.Qt.AscendingOrder
DESCENDING = QtCore.Qt.DescendingOrder

DROP_ENABLED = QtCore.Qt.ItemIsDropEnabled
DRAG_ENABLED = QtCore.Qt.ItemIsDragEnabled
IS_ENABLED = QtCore.Qt.ItemIsEnabled
IS_SELECTABLE = QtCore.Qt.ItemIsSelectable
IS_EDITABLE = QtCore.Qt.ItemIsEditable
IS_CHECKABLE = QtCore.Qt.ItemIsUserCheckable
NO_FLAGS = QtCore.Qt.NoItemFlags
NO_CHILDREN = QtCore.Qt.NoItemFlags

MOVE_ACTION = QtCore.Qt.MoveAction
COPY_ACTION = QtCore.Qt.CopyAction

TEXT_WORD_WRAP = QtCore.Qt.TextWordWrap

CTRL_MOD = QtCore.Qt.ControlModifier

KEY_F11 = QtCore.Qt.Key_F11
KEY_DELETE = QtCore.Qt.Key_Delete



# -*- coding: utf-8 -*-

from typing import Union
import pathlib

from qtpy import QtGui, QtCore

from prettyqt import core


class DesktopServices(QtGui.QDesktopServices):
    @classmethod
    def open_url(cls, location: Union[str, pathlib.Path, QtCore.QUrl]) -> bool:
        return cls.openUrl(core.Url(location))



# -*- coding: utf-8 -*-

from typing import List, Optional
import pathlib

from qtpy import QtCore

from prettyqt.utils import bidict

STANDARD_LOCATIONS = bidict(
    desktop=QtCore.QStandardPaths.DesktopLocation,
    documents=QtCore.QStandardPaths.DocumentsLocation,
    fonts=QtCore.QStandardPaths.FontsLocation,
    applications=QtCore.QStandardPaths.ApplicationsLocation,
    music=QtCore.QStandardPaths.MusicLocation,
    movies=QtCore.QStandardPaths.MoviesLocation,
    pictures=QtCore.QStandardPaths.PicturesLocation,
    temp=QtCore.QStandardPaths.TempLocation,
    home=QtCore.QStandardPaths.HomeLocation,
    cache=QtCore.QStandardPaths.CacheLocation,
    generic_cache=QtCore.QStandardPaths.GenericCacheLocation,
    generic_data=QtCore.QStandardPaths.GenericDataLocation,
    runtime=QtCore.QStandardPaths.RuntimeLocation,
    config=QtCore.QStandardPaths.ConfigLocation,
    download=QtCore.QStandardPaths.DownloadLocation,
    generic_config=QtCore.QStandardPaths.GenericConfigLocation,
    app_data=QtCore.QStandardPaths.AppDataLocation,
    app_local_data=QtCore.QStandardPaths.AppLocalDataLocation,
    app_config=QtCore.QStandardPaths.AppConfigLocation,
)


class StandardPaths(QtCore.QStandardPaths):
    @classmethod
    def get_display_name(cls, location: str) -> str:
        return cls.displayName(STANDARD_LOCATIONS[location])

    @classmethod
    def get_writable_location(cls, location: str) -> Optional[pathlib.Path]:
        path = cls.writableLocation(STANDARD_LOCATIONS[location])
        return pathlib.Path(path) if path else None

    @classmethod
    def get_standard_locations(cls, location: str) -> List[pathlib.Path]:
        paths = cls.standardLocations(STANDARD_LOCATIONS[location])
        return [pathlib.Path(p) for p in paths]



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import core


QtWidgets.QUndoGroup.__bases__ = (core.Object,)


class UndoGroup(QtWidgets.QUndoGroup):
    def __len__(self) -> int:
        return len(self.stacks())

    def __getitem__(self, index) -> QtWidgets.QUndoStack:
        return self.stacks()[index]



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui
from prettyqt.utils import bidict


STYLE_HINTS = bidict(
    any=QtGui.QFont.AnyStyle,
    sans_serif=QtGui.QFont.SansSerif,
    serif=QtGui.QFont.Serif,
    typewriter=QtGui.QFont.TypeWriter,
    decorative=QtGui.QFont.Decorative,
    monospace=QtGui.QFont.Monospace,
    fantasy=QtGui.QFont.Fantasy,
    cursive=QtGui.QFont.Cursive,
    system=QtGui.QFont.System,
)

WEIGHTS = bidict(
    thin=QtGui.QFont.Thin,
    extra_light=QtGui.QFont.ExtraLight,
    light=QtGui.QFont.Light,
    normal=QtGui.QFont.Normal,
    medium=QtGui.QFont.Medium,
    demi_bold=QtGui.QFont.DemiBold,
    bold=QtGui.QFont.Bold,
    extra_bold=QtGui.QFont.ExtraBold,
    black=QtGui.QFont.Black,
)


class Font(QtGui.QFont):
    def __repr__(self):
        return (
            f"Font('{self.family()}', {self.pointSize()}, "
            f"{self.weight()}, {self.italic()})"
        )

    def __getstate__(self):
        return dict(
            family=self.family(),
            pointsize=self.pointSize(),
            weight=self.weight(),
            italic=self.italic(),
        )

    def __setstate__(self, state):
        self.__init__()
        self.setFamily(state["family"])
        if state["pointsize"] > -1:
            self.setPointSize(state["pointsize"])
        self.setWeight(state["weight"])
        self.setItalic(state["italic"])

    @property
    def metrics(self):
        return gui.FontMetrics(self)

    def set_size(self, size: int):
        self.setPointSize(size)

    @classmethod
    def mono(cls, size=8):
        font = cls("Consolas", size)
        # font.setStyleHint()
        return font

    def set_style_hint(self, hint: str):
        """Set the style hint.

        Valid values are "any", "sans_serif", "serif", "typewriter", "decorative",
        "monospace", "fantasy", "cursive", "system"

        Args:
            hint: style hint

        Raises:
            ValueError: invalid style hint
        """
        if hint not in STYLE_HINTS:
            raise ValueError("Invalid style hint")
        self.setStyleHint(STYLE_HINTS[hint])

    def set_weight(self, weight: str):
        """Set the font weight.

        Valid values are "thin", "extra_light", light", "medium", "demi_bold", "bold",
                         "extra_bold", normal", "black"

        Args:
            weight: font weight

        Raises:
            ValueError: invalid font weight
        """
        if weight not in WEIGHTS:
            raise ValueError(f"Invalid weight '{weight}'")
        self.setWeight(WEIGHTS[weight])


if __name__ == "__main__":
    font = Font("Consolas")



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QSpacerItem.__bases__ = (widgets.LayoutItem,)

SIZE_POLICIES = widgets.sizepolicy.SIZE_POLICIES


class SpacerItem(QtWidgets.QSpacerItem):
    def __init__(self, w, h, h_policy, v_policy):
        if h_policy in SIZE_POLICIES:
            h_policy = SIZE_POLICIES[h_policy]
        if v_policy in SIZE_POLICIES:
            v_policy = SIZE_POLICIES[v_policy]
        super().__init__(w, h, h_policy, v_policy)

    def change_size(self, w, h, h_policy="minimum", v_policy="minimum"):
        if h_policy in SIZE_POLICIES:
            h_policy = SIZE_POLICIES[h_policy]
        if v_policy in SIZE_POLICIES:
            v_policy = SIZE_POLICIES[v_policy]
        self.changeSize(w, h, h_policy, v_policy)



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt.utils import colors


class Pen(QtGui.QPen):
    def set_color(self, color: colors.ColorType):
        color = colors.get_color(color)
        self.setColor(color)



# -*- coding: utf-8 -*-
"""Credits to PyQode Authors."""

from typing import Optional

from qtpy import QtWidgets

from prettyqt import core, gui, widgets


class PromptLineEdit(widgets.LineEdit):
    """Extends QLineEdit to show a prompt text and a clear icon."""

    #: Signal emitted when the embedded button is clicked
    clear_clicked = core.Signal()

    def __init__(
        self,
        parent: Optional[QtWidgets.QWidget] = None,
        prompt_text: str = "Search",
        button_icon: gui.icon.IconType = "mdi.delete-circle-outline",
    ):
        super().__init__(parent=parent)
        self._margin = self.sizeHint().height() - 2
        self._spacing = 0
        self._prompt_text = prompt_text
        self.button = widgets.ToolButton(self)
        self.button.set_icon(button_icon)
        with self.button.edit_stylesheet() as ss:
            ss.QToolButton.setValues(border=None, padding="0px")
        self.button.set_cursor("arrow")
        self.button.set_focus_policy("none")
        self.set_button_visible(False)
        self.textChanged.connect(self._on_text_changed)
        self.button.clicked.connect(self.clear)
        self.button.clicked.connect(self.clear_clicked.emit)

    @property
    def prompt_text(self) -> str:
        """Gets/Sets the prompt text."""
        return self._prompt_text

    @prompt_text.setter
    def prompt_text(self, prompt: str):
        self._prompt_text = prompt
        self.update()

    def paintEvent(self, event):
        super().paintEvent(event)

        if self._prompt_text and not self.text() and self.isEnabled():
            option = widgets.StyleOptionFrame()
            self.initStyleOption(option)

            left, top, right, bottom = self.getTextMargins()

            va = self.style().visualAlignment(self.layoutDirection(), self.alignment())
            rect = (
                self.style()
                .subElementRect(widgets.Style.SE_LineEditContents, option, self)
                .adjusted(2, 0, 0, 0)
                .adjusted(left, top, -right, -bottom)
            )
            fm = gui.FontMetrics(self.font())
            text = fm.elided_text(self._prompt_text, mode="right", width=rect.width())
            painter = gui.Painter(self)
            painter.setPen(self.palette().color(gui.Palette.Disabled, gui.Palette.Text))
            painter.drawText(rect, va, text)

    def resizeEvent(self, event):
        # Adjusts Clear button position
        super().resizeEvent(event)
        self.button.resize(core.Size(self._margin, self.height() - 2))
        self.button.move(self.width() - self._margin - 3, 1)

    def set_button_visible(self, visible: bool):
        """Sets the clear button as ``visible``.

        Args:
            visible (bool): Visibility of button
        """
        self.button.setVisible(visible)
        left, top, right, bottom = self.getTextMargins()
        if visible:
            right = self._margin + self._spacing
        else:
            right = 0
        self.setTextMargins(left, top, right, bottom)

    def _on_text_changed(self, text: str):
        """Text changed, update Clear button visibility."""
        self.set_button_visible(len(text) > 0)


if __name__ == "__main__":
    app = widgets.app()
    widget = PromptLineEdit()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Optional
from qtpy import QtCore, QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict


TEXT_DIRECTIONS = bidict(
    top_to_bottom=QtWidgets.QProgressBar.TopToBottom,
    bottom_to_top=QtWidgets.QProgressBar.BottomToTop,
)

ALIGNMENTS = bidict(
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
)


QtWidgets.QProgressBar.__bases__ = (widgets.Widget,)


class ProgressBar(QtWidgets.QProgressBar):
    """Progress dialog.

    wrapper for QtWidgets.QProgressBar
    """

    def __init__(
        self, text_visible: bool = True, parent: Optional[QtWidgets.QWidget] = None
    ):
        super().__init__(parent=parent)
        self.setTextVisible(text_visible)

    def set_alignment(self, alignment: str):
        """Set the alignment of the layout.

        Allowed values are "left", "right", "top", "bottom"

        Args:
            alignment: alignment for the layout

        Raises:
            ValueError: alignment does not exist
        """
        if alignment not in ALIGNMENTS:
            raise ValueError(f"{alignment!r} not a valid alignment.")
        self.setAlignment(ALIGNMENTS[alignment])

    def get_alignment(self) -> str:
        """Return current alignment.

        Possible values: "left", "right", "top", "bottom"

        Returns:
            alignment
        """
        return ALIGNMENTS.inv[self.alignment()]

    def set_text_direction(self, text_direction: str):
        """Set the text direction of the layout.

        Allowed values are "top_to_bottom", "bottom_to_top"

        Args:
            text_direction: text direction for the layout

        Raises:
            ValueError: text direction does not exist
        """
        if text_direction not in TEXT_DIRECTIONS:
            raise ValueError(f"{text_direction!r} not a valid text direction.")
        self.setTextDirection(TEXT_DIRECTIONS[text_direction])

    def get_text_direction(self) -> str:
        """Return current text direction.

        Possible values are "top_to_bottom", "bottom_to_top"

        Returns:
            Text direction
        """
        return TEXT_DIRECTIONS.inv[self.textDirection()]

    def set_range(self, start: int, end: int):
        self.setRange(start, end)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = ProgressBar()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui
from prettyqt.utils import bidict


PRIORITIES = bidict(
    low=QtWidgets.QAction.LowPriority,
    normal=QtWidgets.QAction.NormalPriority,
    high=QtWidgets.QAction.HighPriority,
)

CONTEXTS = bidict(
    widget=QtCore.Qt.WidgetShortcut,
    widget_with_children=QtCore.Qt.WidgetWithChildrenShortcut,
    window=QtCore.Qt.WindowShortcut,
    application=QtCore.Qt.ApplicationShortcut,
)


QtWidgets.QAction.__bases__ = (core.Object,)


class Action(QtWidgets.QAction):
    def __init__(
        self,
        parent=None,
        text: str = "",
        icon: gui.icon.IconType = None,
        shortcut: Optional[str] = None,
        tooltip: str = "",
    ):
        super().__init__(parent)
        self.set_text(text)
        self.set_icon(icon)
        self.set_shortcut(shortcut)
        self.set_tooltip(tooltip)

    def serialize_fields(self):
        return dict(
            text=self.text(),
            enabled=self.isEnabled(),
            shortcut=self.shortcut(),
            tooltip=self.toolTip(),
            checkable=self.isCheckable(),
            checked=self.isChecked(),
            priority=self.get_priority(),
            shortcut_context=self.get_shortcut_context(),
            statustip=self.statusTip(),
        )

    def __setstate__(self, state):
        self.__init__()
        self.set_text(state.get("text", ""))
        self.set_enabled(state.get("enabled", True))
        self.set_shortcut(state["shortcut"])
        self.set_tooltip(state.get("tooltip", ""))
        self.set_statustip(state.get("statustip", ""))
        self.set_checked(state.get("checked", False))
        self.set_priority(state["priority"])
        self.set_shortcut_context(state["shortcut_context"])
        self.set_checkable(state["checkable"])

    def set_text(self, text: str):
        self.setText(text)

    def set_enabled(self, enabled: bool = True):
        self.setEnabled(enabled)

    def set_disabled(self):
        self.setEnabled(False)

    def set_tooltip(self, text: str):
        self.setToolTip(text)

    def set_statustip(self, text: str):
        self.setStatusTip(text)

    def set_checked(self, value: bool):
        self.setChecked(value)

    def set_checkable(self, value: bool):
        self.setCheckable(value)

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)

    def set_shortcut(self, shortcut):
        if shortcut:
            self.setShortcut(shortcut)

    def set_menu(self, menu):
        self.setMenu(menu)

    def set_priority(self, priority: str):
        """Set priority of the action.

        Allowed values are "low", "normal", "high"

        Args:
            priority: priority for the action

        Raises:
            ValueError: priority does not exist
        """
        if priority not in PRIORITIES:
            raise ValueError(f"{priority} not a valid priority.")
        self.setPriority(PRIORITIES[priority])

    def get_priority(self) -> str:
        """Return current priority.

        Possible values: "low", "normal", "high"

        Returns:
            priority
        """
        return PRIORITIES.inv[self.priority()]

    def set_shortcut_context(self, context: str):
        """Set shortcut context.

        Allowed values are "widget", "widget_with_children", "window", "application"

        Args:
            context: shortcut context

        Raises:
            ValueError: shortcut context does not exist
        """
        if context not in CONTEXTS:
            raise ValueError(f"{context} not a valid shortcut context.")
        self.setShortcutContext(CONTEXTS[context])

    def get_shortcut_context(self) -> str:
        """Return shortcut context.

        Possible values: "widget", "widget_with_children", "window", "application"

        Returns:
            shortcut context
        """
        return CONTEXTS.inv[self.shortcutContext()]

    def show_shortcut_in_contextmenu(self, state: bool = True):
        self.setShortcutVisibleInContextMenu(state)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    action = Action("This is a test")
    app.exec_()



# -*- coding: utf-8 -*-

"""Multimedia module."""

from .mediabindableinterface import MediaBindableInterface
from .videoencodersettings import VideoEncoderSettings
from .audioencodersettings import AudioEncoderSettings
from .mediaobject import MediaObject
from .mediacontent import MediaContent
from .mediacontrol import MediaControl
from .mediaplaylist import MediaPlaylist
from .mediaplayer import MediaPlayer
from .mediarecorder import MediaRecorder
from .audiorecorder import AudioRecorder

__all__ = [
    "MediaBindableInterface",
    "VideoEncoderSettings",
    "AudioEncoderSettings",
    "MediaControl",
    "MediaContent",
    "MediaObject",
    "MediaPlaylist",
    "MediaPlayer",
    "MediaRecorder",
    "AudioRecorder",
]



# -*- coding: utf-8 -*-

"""Top-level package for PrettyQt."""

__author__ = """Philipp Temminghoff"""
__email__ = "phil65@kodi.tv"
__version__ = "0.109.0"


__all__ = [
    "core",
    "gui",
    "widgets",
    "custom_widgets",
    "custom_models",
    "syntaxhighlighters",
    "charts",
    "webenginewidgets",
]



# -*- coding: utf-8 -*-

"""Charts module."""

from .abstractseries import AbstractSeries
from .xyseries import XYSeries
from .scatterseries import ScatterSeries
from .lineseries import LineSeries
from .chart import Chart
from .chartview import ChartView

__all__ = [
    "AbstractSeries",
    "XYSeries",
    "ChartView",
    "Chart",
    "LineSeries",
    "ScatterSeries",
]
